/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __FOC_UI_UART_H__
#define __FOC_UI_UART_H__
/*!
* @file foc_ui_uart.h
*
* @brief This file provides UART communication with user interface of host computer functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac780x.h"
#include "ac780x_uart_reg.h"
#include "globalvar.h"

/* ============================================  Define  ============================================ */
#define TOOL_UART_FUNCTION                      /*!< If this macro definition is defined, the Host PC tool UART function will be enabled, otherwise it will be closed */
#define TOOL_UART_BAUDRATE      230400          /*!< UART communication baud rate */

#define UI_REG8_BUF_SIZE        36              /*!< UART register buffer size, unit: byte */
#define UI_REG16_BUF_SIZE       205             /*!< UART register buffer size, unit: 2 byte */
#define UI_REG32_BUF_SIZE       15              /*!< UART register buffer size, unit: 4 byte */

#define RW_BUF_SIZE             12              /*!< Master read and write communication buffer size, unit: byte */
#define IDENTIFY_BUF_SIZE       9               /*!< Identify RX buffer size, unit: byte */

#define RX_BUF_SIZE             256             /*!< UART RX buffer size, unit: byte */
#define TX_BUF_SIZE             256             /*!< UART TX buffer size, unit: byte */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Enumeration of motor control modes.
*/
typedef enum {
    REV0 = 0,                                   /*!< Reserve */
    REV1,                                       /*!< Reserve */
    SQUARE_HALL,                                /*!< BLDC controlled with Hall sensor */
    SQUARE_SENSORLESS,                          /*!< BLDC controlled with sensorless */
    SINE_HALL,                                  /*!< FOC with Hall sensor */
    SINE_HALL_ENCODER,                          /*!< FOC with Hall and Encoder sensors */
    SINE_SENSORLESS,                            /*!< FOC with sensorless */
    SINE_ENCODER                                /*!< FOC with Encoder sensor */
} MOTOR_MAIN_MODE;

/*!
* @brief Enumeration of sensorless FOC subitems.
*/
typedef enum {
    DEFAULT = 0,                                /*!< Default */
    SMO_ATAN,                                   /*!< Sliding mode observer algorithm with obtaining angle by arctan method */
    SMO_PLL,                                    /*!< Sliding mode observer algorithm with obtaining angle by PLL method */
    FLUX_ATAN,                                  /*!< Flux observer algorithm with obtaining angle by arctan method */
    FLUX_PLL                                    /*!< Flux observer algorithm with obtaining angle by PLL method */
} SINE_SENSORLESS_SUB_MODE;

/*!
* @brief UART buffer parameters structure.
*/
typedef struct
{
    uint16_t uartBufSize;                       /*!< UART buffer fifo size */
    uint16_t uartBufWrIndex;                    /*!< UART buffer fifo write ID */
    uint16_t uartBufRdIndex;                    /*!< UART buffer fifo read ID */
    uint16_t uartBufCnt;                        /*!< UART buffer data count */
    uint8_t  uartBufOvf;                        /*!< UART buffer fifo overflow flag */
    uint8_t  *uartBufAddr;                      /*!< UART buffer base address */
} UART_BUF;

/*!
* @brief System operation states structure.
*/
typedef struct
{
    uint8_t communicateStatus;                  /*!< Communication success or not */
    uint8_t getVersionStatus;                   /*!< Get version success or not */
    uint8_t startMotorStatus;                   /*!< Start motor success or not */
    uint8_t stopMotorStatus;                    /*!< Stop motor success or not */
    uint8_t stopRampStatus;                     /*!< Stop ramp success or not */
    uint8_t identifyStatus;                     /*!< Parameter identification success or not */
    uint8_t cmdProcStatus;                      /*!< Command process status 0x00(0%) -- 0x64(100%) */
    uint8_t identifyOffStatus;                  /*!< Turn off parameter identification success or not */
    uint8_t clearFaultStatus;                   /*!< Clear fault success or not */
    uint8_t readRegStatus;                      /*!< Read register success or not */
    uint8_t writeRegStatus;                     /*!< Write register success or not */
    uint8_t checkeErrStatus;                    /*!< Command check error status */
    uint8_t startHallSelfLearnStatus;           /*!< Start Hall self learning success or not */
    uint8_t stopHallSelfLearnStatus;            /*!< Stop Hall self learning success or not */
} CMD_PROC_STATUS;

/*!
* @brief UART read and write parameters structure.
*/
typedef struct
{
    uint8_t motorID;                            /*!< Save motor ID from master command */
    uint8_t uiRdBuf[RW_BUF_SIZE];               /*!< Save data length and data ID from read register command */
    uint8_t uiWrBuf[RW_BUF_SIZE];               /*!< Save data length and data ID from write register command */
    uint8_t i8_num;                             /*!< Save read/write one byte register ID numbers */
    uint8_t i16_num;                            /*!< Save read/write two byte register ID numbers */
    uint8_t i32_num;                            /*!< Save read/write four byte register ID numbers */
    uint8_t WriteSpeedValid;                    /*!< Write speed valid flag */
    uint8_t uiIdentifyBuf[IDENTIFY_BUF_SIZE];   /*!< Save motor parameter from identification command */
    uint8_t uiCheckCmd;                         /*!< Save UI command while checksum error */

    uint8_t uartTimeOutCntEn;                   /*!< Enable time count while UART process a package data */
    uint16_t uartTimeOutCnt;                    /*!< Count time in UART process a package data, unit: ms */
    uint8_t cmdProcTimeOutCntEn;                /*!< Enable time count while execute master command */
    uint16_t cmdProcTimeOutCnt;                 /*!< Count time in execute master command, unit: ms */
    uint8_t cmdReadCnt;                         /*!< Count receive read command times */
} CMD_PROC_VAR;

/*!
* @brief UART parameters transmition structure.
*/
typedef struct
{
    uint16_t appMode;                           /*!< Motor algorithm mode */
    uint16_t heatTemperature;                   /*!< Motor temperature */
    uint8_t dacOut1;                           /*!< DAC output id in channel 1 */
    uint8_t dacOut2;                           /*!< DAC output id in channel 2 */
    uint16_t appVerYear;                        /*!< App version year */
    uint16_t appVerDate;                        /*!< App version month and date */
    uint16_t appVerMain;                        /*!< App main version */
    uint16_t appVerRelease;                     /*!< App release version */
    uint16_t libVerYear;                        /*!< Foc lib version year */
    uint16_t libVerDate;                        /*!< Foc lib version month and date */
    uint16_t libVerMain;                        /*!< Foc lib main version */
    uint16_t libVerRelease;                     /*!< Foc lib release version */
} UI_COMMUNICATION_PARAM;

/* ==========================================  Variables  =========================================== */

extern UART_BUF g_uiRxBuf;                      /*!< UART receive buffer struct */
extern UART_BUF g_uiTxBuf;                      /*!< Define UI uart interactive tx buffer struct */

extern UI_COMMUNICATION_PARAM g_uiCommParam;    /*!< UI Register param table struct */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Tool Uart peripheral initialization.
*
* @param[in] none
* @return none
*/
void UI_ToolUart_Init(void);

/*!
* @brief Uart communication register table initialization.
*
* @param[in] none
* @return none
*/
void UI_RegBuf_Init(void);

/*!
* @brief Uart process command recieved from master according to communication protocol.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_ToolCmd_Proc(UART_BUF *uiRxBuf);

/*!
* @brief Uart communication consume time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_UART_TimeOut_Cnt(void);

/*!
* @brief master command process time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_MasterCmd_ProcCnt(void);

/*!
* @brief UART receive interrrupt isr,occur when receice a char from master.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_RxISR_Task(UART_BUF *uartRx);

/*!
* @brief UART transmit interrrupt isr,occur when transmit a char to master.
*
* @param[in] uartTx: pointer to UART_BUF structure, store the data send out to master
* @return none
*/
void UI_UART_TxISR_Task(UART_BUF *uartTx);

/* ======================================  Functions define  ======================================== */

#endif /* __FOC_UI_UART_H__ */

/* =============================================  EOF  ============================================== */
