/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_mcu_config.c
*
* @brief This file provides MCU peripheral initialization functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac780x.h"
#include "ac780x_gpio_reg.h"
#include "ac780x_pwdt_reg.h"
#include "ac780x_ctu_reg.h"
#include "ac780x_acmp_reg.h"
#include "ac780x_dma_reg.h"
#include "ac780x_spi_reg.h"
#include "string.h"
#include "bldc_mcu_config.h"
#include "bldc_isr.h"
#include "bldc_app.h"
#include "bldc_keys.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
uint32_t g_ADCRegularBuffer[10] = {0};  /*!< Buffer to storage Regular Group ADC Value */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief BLDC Control Module GPIO Initialize.
*
* @param[in] none
* @return none
*/
void MC_GPIO_Initialize(void)
{
    /* Setting the multiplexing function of GPIO pin */
    GPIO_SetFunc(GPIOC, GPIO_PIN9, GPIO_FUN0);
    /* Setting output as the attribute of GPIO pin */
    GPIO_SetDir(GPIOC, GPIO_PIN9, GPIO_OUT);

    GPIO_SetFunc(GPIOC, GPIO_PIN5, GPIO_FUN0);
    /* Setting output as the attribute of GPIO pin */
    GPIO_SetDir(GPIOC, GPIO_PIN5, GPIO_OUT);

    /* Setting the LED2 to display operation states */
    GPIO_SetFunc(GPIOA, GPIO_PIN6, GPIO_FUN0);
    GPIO_SetDir(GPIOA, GPIO_PIN6, GPIO_OUT);
}

/*!
* @brief PWM Initialization for debugging, PWM module can be used for debugging output.
*
* @param[in] PWMx: PWM Module
* @return none
*/
void MC_DEBUG_PWM_Initialize(PWM_Type *PWMx)
{
    PWM_SimplyConfigType SimplyConfig;

    /*
    * GPIO Configuration for debug PWM
    * PB12    --> PWM0_CH2
    * PB11    --> PWM0_CH3
    */
    /* PWM0_CH2, Debug channel 0 */
    GPIO_SetFunc(GPIOB, GPIO_PIN12, GPIO_FUN1);
    /* PWM0_CH3, Debug channel 1 */
    GPIO_SetFunc(GPIOB, GPIO_PIN11, GPIO_FUN1);

    /* Config DEBUG-PWM Interface */
    memset(&SimplyConfig, 0, sizeof(SimplyConfig));
    SimplyConfig.allChCombineMode       = PWM_INDEPENDENT_MODE;
    SimplyConfig.countMode              = PWM_UP_COUNT;
    /* PWM_HIGH_TRUE */
    SimplyConfig.levelMode              = PWM_HIGH_TRUE;
    /* PWM_CLK_SOURCE_BUS */
    SimplyConfig.clkSource              = PWM_CLK_SOURCE_APB;
    SimplyConfig.clkPsc                 = 0;
    SimplyConfig.initValue              = 0;
    SimplyConfig.maxValue               = 4096;
    SimplyConfig.chValue[0]             = 0;
    SimplyConfig.chValue[2]             = 0;
    SimplyConfig.chValue[4]             = 0;
    SimplyConfig.interruptEn            = DISABLE;
    SimplyConfig.callBack               = NULL;
    PWM_SimplyInit(PWMx, &SimplyConfig);
}

/*!
* @brief Calbiration ADC to get adc channel offset value.
*
* @param[in] sequencechannel : ADC Inject Group sequence num, 0-3
* @return dcoffset: ADC Inject Group channel offset
*/
int16_t MC_ADC_Calbiration(uint8_t sequencechannel)
{
    uint16_t index = 0;
    uint32_t sumOffset =0;
    int16_t adcOffset = 0;

    ADC_SetRegularTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);
    ADC_SetInjectTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);

    ADC_ClearIEOCFlag(ADC0);
    udelay(10);

    for (index = 0; index < NUM_OF_CONVERSIONS; index ++)
    {
        ADC_SoftwareStartInjectConvert(ADC0);
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
        udelay(10);
        ADC_SoftwareStartInjectConvert(ADC0);
#endif
        while (!ADC_GetIEOCFlag(ADC0));

        sumOffset += ADC_GetInjectData(ADC0, sequencechannel);

        ADC_ClearIEOCFlag(ADC0);
    }
    adcOffset = sumOffset / NUM_OF_CONVERSIONS;

    return adcOffset;
}

/*!
* @brief Timer0 Initialize, generates 1ms interrupt for bldc control time base.
*
* @param[in] none
* @return none
*/
void BLDC_TIM_Initialize(void)
{
    TIMER_ConfigType timerConfig = {0};
    memset(&timerConfig, 0, sizeof(TIMER_ConfigType));
    /* config 1ms delay*/
    timerConfig.periodValue = TIMER_1MS;
    timerConfig.interruptEn = ENABLE;
    timerConfig.linkModeEn = DISABLE;
    timerConfig.timerEn = ENABLE;
    timerConfig.callBack = BLDC_Base_TimerCallback;

    NVIC_SetPriority(TIMER_CHANNEL0_IRQn, 3);
    TIMER_Init(TIMER_CHANNEL0, &timerConfig);
}

/*!
* @brief Timer1 Initialize, generates interrupt to check Bemf over zero point task in bldc sebsorless control.
*
* @param[in] none
* @return none
*/
void BLDC_Bemf_TimerInit(void)
{
    TIMER_ConfigType timerConfig = {0};
    memset(&timerConfig, 0, sizeof(TIMER_ConfigType));

    timerConfig.periodValue = COMMUTATION_TIMER_PERIOD;
    timerConfig.interruptEn = ENABLE;
    timerConfig.linkModeEn = DISABLE;
    timerConfig.timerEn = ENABLE;
    timerConfig.callBack = Timer1_Callback;

    TIMER_Init(TIMER_CHANNEL1, &timerConfig);
    NVIC_SetPriority(TIMER_CHANNEL1_IRQn, 1);
}

/*!
* @brief CTU Initialize, config ADC sampling trigger mode and trigger time in bldc control.
*
* @param[in] none
* @return none
*/
void BLDC_CTU_Init(void)
{
    CTU_ConfigType ctuConfig;
    memset(&ctuConfig, 0, sizeof(CTU_ConfigType));

#if (defined GATE_DRIVER_HIGH_HIGH)
    ctuConfig.clkPsc = CTU_CLK_PRESCALER_1;
    ctuConfig.adcInjectTriggerSource = CTU_TRIGGER_ADC_PWM1_MATCH;
    ctuConfig.delay1Time = 0;
#else
    /* NULL */
#endif

    CTU_Init(&ctuConfig);
}

/*!
* @brief DMA Initialize in ADC config, config DMA transfer method and address for ADC regular group.
*
* @param[in] none
* @return none
*/
void BLDC_ADCDMA_Init(void)
{
    DMA_ChannelType *DMAx = DMA0_CHANNEL0;
    uint32_t tmpMemStartAddr = (uint32_t)&g_ADCRegularBuffer[0];
    uint32_t tmpMemEndAddr = (uint32_t)&g_ADCRegularBuffer[9];
    DMA_ConfigType tmpDMAConfig;
    memset(&tmpDMAConfig, 0x00, sizeof(DMA_ConfigType));

    tmpDMAConfig.memStartAddr = tmpMemStartAddr;
    tmpDMAConfig.memEndAddr = tmpMemEndAddr;
    tmpDMAConfig.periphStartAddr = (uint32_t)(&(ADC0->RDR));
    tmpDMAConfig.channelEn = ENABLE;
    tmpDMAConfig.finishInterruptEn = DISABLE;
    tmpDMAConfig.halfFinishInterruptEn = DISABLE;
    tmpDMAConfig.errorInterruptEn = DISABLE;
    tmpDMAConfig.channelPriority = DMA_PRIORITY_VERY_HIGH;
    tmpDMAConfig.circular = ENABLE;
    tmpDMAConfig.direction = DMA_READ_FROM_PERIPH;
    tmpDMAConfig.MEM2MEM = DISABLE;
    tmpDMAConfig.memByteMode = DMA_MEM_BYTE_MODE_1TIME;
    tmpDMAConfig.memIncrement = ENABLE;
    tmpDMAConfig.periphIncrement = DISABLE;
    tmpDMAConfig.memSize = DMA_MEM_SIZE_32BIT;
    tmpDMAConfig.periphSize = DMA_PERIPH_SIZE_16BIT;
    tmpDMAConfig.transferNum = 8;
    tmpDMAConfig.periphSelect = DMA_PEPIRH_ADC0;

    DMA_Init(DMAx, &tmpDMAConfig);
}

/*!
* @brief Get ADC current sampling channel offset value.
*
* @param[in] none
* @return none
*/
void BLDC_ADC_Calbiration(void)
{
    g_bldc_adSample.busCurOffset = MC_ADC_Calbiration(0U);
#if (defined BEMF_DETECT_SOFTWARE)
    g_bldc_adSample.phaseVoltageOffsetA = MC_ADC_Calbiration(1U);
    g_bldc_adSample.phaseVoltageOffsetB = MC_ADC_Calbiration(2U);
    g_bldc_adSample.phaseVoltageOffsetC = MC_ADC_Calbiration(3U);
#endif
}

/*!
* @brief ADC Initialize, config ADC mode, sampling channel, trigger source, and DMA transfer.
*
* @param[in] none
* @return none
*/
void BLDC_ADC_Init(void)
{
    ADC_ConfigType tempAdcConfig;
    ADC_ConfigType* adcConfig;
    adcConfig = &tempAdcConfig;
    memset(adcConfig, 0x00, sizeof(ADC_ConfigType));

    /**
    GPIO Configuration for ADC
    PA3     --> ADC_IN7
    PA2     --> ADC_IN8
    PC1     --> ADC_IN9
    PC0     --> ADC_IN10
    */

    /* ADC channel 10, Control keys(Star_Stop/Fast/Slow/Direction) */
    GPIO_SetFunc(GPIOC, GPIO_PIN0, GPIO_FUN2);
    /* ADC channel 7, I_Bus */
    GPIO_SetFunc(GPIOA, GPIO_PIN3, GPIO_FUN2);
    /* ADC channel 8, VBus */
    GPIO_SetFunc(GPIOA, GPIO_PIN2, GPIO_FUN2);
    /* ADC channel 9, variohm adjust AD */
    GPIO_SetFunc(GPIOC, GPIO_PIN1, GPIO_FUN2);
#if (defined BEMF_DETECT_SOFTWARE)
    /* ADC channel 0,1,2, BEMF A,B,C */
    GPIO_SetFunc(GPIOA, GPIO_PIN11, GPIO_FUN2);
    GPIO_SetFunc(GPIOA, GPIO_PIN10, GPIO_FUN2);
    GPIO_SetFunc(GPIOA, GPIO_PIN9, GPIO_FUN2);
#endif

    CKGEN_Enable(CLK_ADC0, ENABLE);

    adcConfig->scanModeEn = ENABLE;
    adcConfig->continousModeEn = ENABLE;
    adcConfig->regularDiscontinousModeEn = DISABLE;
    adcConfig->injectDiscontinousModeEn = DISABLE;
    adcConfig->injectAutoModeEn = DISABLE;
    adcConfig->intervalModeEn = DISABLE;

#if (defined SYS_CLK_72M)
    adcConfig->clkPsc = ADC_CLK_PRESCALER_2;
#elif (defined SYS_CLK_48M)
    adcConfig->clkPsc = ADC_CLK_PRESCALER_1;
#endif
    adcConfig->regularSequenceLength = 4;
#if (defined BEMF_DETECT_SOFTWARE)
    adcConfig->injectSequenceLength = 4;
#else
    adcConfig->injectSequenceLength = 1;
#endif
    adcConfig->regularDiscontinousNum = 0;
    adcConfig->EOCInterruptEn = DISABLE;
    adcConfig->IEOCInterruptEn = DISABLE;
    adcConfig->interruptEn = DISABLE;
    adcConfig->regularDMAEn = ENABLE;
    adcConfig->callBack = BLDC_ADC_Callback;
    adcConfig->dataAlign = ADC_DATA_ALIGN_RIGHT;
    adcConfig->powerMode = ADC_POWER_ON;
    adcConfig->regularTriggerMode = ADC_TRIGGER_INTERNAL;
    adcConfig->injectTriggerMode = ADC_TRIGGER_INTERNAL;

    ADC_Init(ADC0, adcConfig);
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_8, AD_SAMP_CLK, 0);
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_9, AD_SAMP_CLK, 1);
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_10, AD_SAMP_CLK, 2);
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_7, AD_SAMP_CLK, 3);
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_7, AD_SAMP_CLK, 0);
#if (defined BEMF_DETECT_SOFTWARE)
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_0, AD_SAMP_CLK, 1);
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_1, AD_SAMP_CLK, 2);
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_2, AD_SAMP_CLK, 3);
#endif

    /* Calbiration Ibus Offset */
    BLDC_ADC_Calbiration();

    ADC_SetRegularTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);
    ADC_SetInjectTriggerSource(ADC0, ADC_TRIGGER_EXTERNAL);

    BLDC_ADCDMA_Init();
    ADC_SoftwareStartRegularConvert(ADC0);
    ADC_SetIEOCInterrupt(ADC0, ENABLE);

    NVIC_SetPriority(ADC0_IRQn, 1);
}

/*!
* @brief PWM sync mode initialize, config PWM sync mode.
*
* @param[in] none
* @return none
*/
void PWM_SyncModeSet(void)
{
    PWM_SyncConfigType syncConfig;

    syncConfig.syncTriggerMode = PWM_SYNC_TRIGGER_SOFTWARE;
    syncConfig.hardwareSync0En = DISABLE;
    syncConfig.hardwareSync1En = DISABLE;
    syncConfig.hardwareSync2En = DISABLE;
    syncConfig.autoClearHWTriggerEn = DISABLE;
    syncConfig.counterInitSyncEn = ENABLE;
    syncConfig.outCtrlSyncEn = DISABLE;
    syncConfig.inverterSyncEn = DISABLE;
    syncConfig.outmaskSyncEn = DISABLE;
    syncConfig.polaritySyncEn = DISABLE;
    syncConfig.chValueSyncEn[0] = ENABLE;
    syncConfig.chValueSyncEn[1] = ENABLE;
    syncConfig.chValueSyncEn[2] = ENABLE;
    syncConfig.chValueSyncEn[3] = ENABLE;
    syncConfig.maxLoadingPointEn = DISABLE;
    syncConfig.minLoadingPointEn = ENABLE;
    syncConfig.syncEn = ENABLE;

    PWM_InitSyncControl(PWM1, &syncConfig);
}

/*!
* @brief PWM Initialize, config PWM mode, polarity, and FAULT interrupt.
*
* @param[in] none
* @return none
*/
void BLDC_PWM_Initialize(void)
{
    static PWM_CombineChConfig chConfig[4];
    static PWM_ModulationConfigType pwmConfig;
    static PWM_ConfigType config;
    static PWM_FaultConfig faultConfig;
    static PWM_SyncConfigType syncConfig;

    /**
    GPIO Configuration for PWM1
    PB5     --> PWM1_IN0
    PB4     --> PWM1_IN1
    PB8     --> PWM1_IN2
    PB7     --> PWM1_IN3
    PB10    --> PWM1_IN4
    PB9     --> PWM1_IN5
    PB6     --> PWM1_IN6
    */

    /* PWM output channel 0 */
    GPIO_SetFunc(GPIOB, GPIO_PIN5, GPIO_FUN1);
    /* PWM output channel 1 */
    GPIO_SetFunc(GPIOB, GPIO_PIN4, GPIO_FUN1);
    /* PWM output channel 2 */
    GPIO_SetFunc(GPIOB, GPIO_PIN8, GPIO_FUN1);
    /* PWM output channel 3 */
    GPIO_SetFunc(GPIOB, GPIO_PIN7, GPIO_FUN1);
    /* PWM output channel 4 */
    GPIO_SetFunc(GPIOB, GPIO_PIN10, GPIO_FUN1);
    /* PWM output channel 5 */
    GPIO_SetFunc(GPIOB, GPIO_PIN9, GPIO_FUN1);

    /* PWM hardware current overflow fault */
    GPIO_SetFunc(GPIOB, GPIO_PIN6, GPIO_FUN2);

    memset(chConfig, 0, sizeof(chConfig));
    memset(&pwmConfig, 0, sizeof(PWM_ModulationConfigType));
    memset(&faultConfig, 0, sizeof(PWM_FaultConfig));
    memset(&syncConfig, 0, sizeof(PWM_SyncConfigType));
    memset(&config, 0, sizeof(PWM_ConfigType));
    CKGEN_Enable(CLK_PWM1, ENABLE);

#if (defined GATE_DRIVER_HIGH_HIGH)
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_0, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_1, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_2, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_3, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_4, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_5, PWM_LOW_LEVEL);
#elif (defined GATE_DRIVER_HIGH_LOW)
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_0, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_1, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_2, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_3, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_4, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_5, PWM_HIGH_LEVEL);
#elif (defined GATE_DRIVER_LOW_HIGH)
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_0, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_1, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_2, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_3, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_4, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_5, PWM_LOW_LEVEL);
#elif (defined GATE_DRIVER_LOW_LOW)
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_0, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_1, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_2, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_3, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_4, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_5, PWM_HIGH_LEVEL);
#endif
    /* channel 0/1 */
    chConfig[0].pairChannel = PWM_CH_0;
    chConfig[0].ch1stValue = 0;
    chConfig[0].ch2ndValue = 0;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].levelMode = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].levelMode = PWM_LOW_TRUE;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].levelMode = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].levelMode = PWM_LOW_TRUE;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].levelMode = PWM_LOW_TRUE;
    #endif
#else

#endif
    chConfig[0].deadtimeEn = ENABLE;
    chConfig[0].complementEn = ENABLE;
    chConfig[0].ch1stMatchDir = PWM_MATCH_DIR_UP;
    chConfig[0].ch2ndMatchDir = PWM_MATCH_DIR_DOWN;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#else

#endif

    /* channel 2/3 */
    chConfig[1].pairChannel = PWM_CH_2;
    chConfig[1].ch1stValue = 0;
    chConfig[1].ch2ndValue = 0;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].levelMode = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].levelMode = PWM_LOW_TRUE;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].levelMode = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].levelMode = PWM_LOW_TRUE;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].levelMode = PWM_LOW_TRUE;
    #endif
#else

#endif
    chConfig[1].deadtimeEn = ENABLE;
    chConfig[1].complementEn = ENABLE;
    chConfig[1].ch1stMatchDir = PWM_MATCH_DIR_UP;
    chConfig[1].ch2ndMatchDir = PWM_MATCH_DIR_DOWN;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#else

#endif

    /* channel 4/5 */
    chConfig[2].pairChannel = PWM_CH_4;
    chConfig[2].ch1stValue = 0;
    chConfig[2].ch2ndValue = 0;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].levelMode = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].levelMode = PWM_LOW_TRUE;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].levelMode = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].levelMode = PWM_LOW_TRUE;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].levelMode = PWM_LOW_TRUE;
    #endif
#else

#endif
    chConfig[2].deadtimeEn = ENABLE;
    chConfig[2].complementEn = ENABLE;
    chConfig[2].ch1stMatchDir = PWM_MATCH_DIR_UP;
    chConfig[2].ch2ndMatchDir = PWM_MATCH_DIR_DOWN;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#else

#endif

#if (defined GATE_DRIVER_HIGH_HIGH)
    chConfig[3].pairChannel = PWM_CH_6;
    chConfig[3].ch1stValue = (PWM_PERIOD_VALUE >> 1);
    chConfig[3].ch2ndValue = 0;
    chConfig[3].levelMode = PWM_HIGH_TRUE;
    chConfig[3].deadtimeEn = DISABLE;
    chConfig[3].complementEn = DISABLE;
    chConfig[3].ch1stMatchDir = PWM_MATCH_DIR_UP;
    chConfig[3].ch2ndMatchDir = PWM_MATCH_DIR_UP;
    chConfig[3].ch1stTriggerEn = ENABLE;
    chConfig[3].ch2ndTriggerEn = DISABLE;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif

#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif

#elif (defined GATE_DRIVER_LOW_LOW)
    /* NULL */
#else

#endif

    /* modulation mode config */
#if (defined GATE_DRIVER_HIGH_HIGH)
    pwmConfig.initTriggerEn = DISABLE;
    pwmConfig.combineChannelNum = 4;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode = PWM_UP_COUNT;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    /* NULL */

#elif (defined GATE_DRIVER_LOW_HIGH)
    /* NULL */

#elif (defined GATE_DRIVER_LOW_LOW)
    /* NULL */
#else

#endif
    pwmConfig.combineChConfig = chConfig;
    pwmConfig.deadtime = PWM_DEAD_TIME;
    pwmConfig.deadtimePsc = PWM_DEADTIME_DIVID_4;

    /* pwm config */
    config.mode = PWM_MODE_MODULATION;
    config.initModeStruct = &pwmConfig;
    config.clkSource = PWM_CLK_SOURCE_NONE;
    config.clkPsc = PWM_PRSC;
    config.maxValue = PWM_PERIOD_VALUE;
    config.interruptEn = ENABLE;
    config.callBack = BLDC_PWM_FaultCallback;
    PWM_Init(PWM1, &config);

    /* pwm sync config */
    PWM_SyncModeSet();

    /* fault control config */
    faultConfig.channelConfig[PWM_FAULT_CH_1].faultInputEn = ENABLE;
    faultConfig.channelConfig[PWM_FAULT_CH_1].faultFilterEn = DISABLE;
    faultConfig.channelConfig[PWM_FAULT_CH_1].faultPolarity = OVERCUR_FEEDBACK_POLARITY;
    faultConfig.mode = PWM_FAULT_CTRL_MANUAL_ALL;
    faultConfig.filterValue = 0;
    faultConfig.faultCtrlOutputEn[0] = ENABLE;
    faultConfig.faultCtrlOutputEn[1] = ENABLE;
    faultConfig.faultCtrlOutputEn[2] = ENABLE;
    faultConfig.faultCtrlOutputEn[3] = ENABLE;
    faultConfig.interruptEn = ENABLE;
    PWM_InitFaultControl(PWM1, &faultConfig);

    NVIC_SetPriority(PWM1_IRQn, 0);
    PWM_SetClockSource(PWM1, PWM_CLK_SOURCE_APB);
}

/*!
* @brief ACMP Initialize, config ACMP mode and positive,negative compare channel.
*
* @param[in] none
* @return none
*/
void BLDC_Bemf_ACMP_Init(void)
{
    ACMP_ConfigType acmpConfig;
    memset(&acmpConfig, 0, sizeof(ACMP_ConfigType));

    /**
    GPIO Configuration for ACMP
    PA11    --> ADC_IN0
    PA10    --> ADC_IN1
    PA9     --> ADC_IN2
    PA8     --> ADC_IN3
    */

    /* ADC channel 0, Bemf_U */
    GPIO_SetFunc(GPIOA, GPIO_PIN11, GPIO_FUN2);
    /* ADC channel 1, Bemf_V */
    GPIO_SetFunc(GPIOA, GPIO_PIN10, GPIO_FUN2);
    /* ADC channel 2, Bemf_W */
    GPIO_SetFunc(GPIOA, GPIO_PIN9, GPIO_FUN2);
    /* ADC channel 3, Bemf_ref */
    GPIO_SetFunc(GPIOA, GPIO_PIN8, GPIO_FUN2);

    /* ACMP Polling Mode Configuration */
    acmpConfig.pollingModeType = ACMP_NONE_POLLING;

    acmpConfig.pollingSequence = 7;
    acmpConfig.pollingDiv = ACMP_CLK_DIVIDE_50;
    acmpConfig.hallOutEn = ENABLE;
    acmpConfig.hallAOutputCh = ACMP_EXTERNAL_CH0;
    acmpConfig.hallBOutputCh = ACMP_EXTERNAL_CH1;
    acmpConfig.hallCOutputCh = ACMP_EXTERNAL_CH2;

    /* ACMP Init Configuration */
    acmpConfig.negativePin = ACMP_EXTERNAL_CH3;
    acmpConfig.edgeType = ACMP_EDGE_TRIGGER_FALLING_RISING;
    acmpConfig.dacRef = ACMP_DAC_BANDGAP;
    acmpConfig.interruptEn = DISABLE;
    acmpConfig.acmpEn = ENABLE;

    acmpConfig.callBack = NULL;
    ACMP_Init(ACMP0, &acmpConfig);
    NVIC_SetPriority(ACMP0_IRQn, 1);
}

/*!
* @brief PWDT Initialize in bldc sensorless control, config PWDT mode and PWDT interrupt.
*
* @param[in] none
* @return none
*/
void BLDC_Bemf_PWDT_Initialize(void)
{
    PWDT_ConfigType  pwdtConfig;
    memset(&pwdtConfig, 0, sizeof(PWDT_ConfigType));

    /**
    GPIO Configuration for PWDT1
    PC8     --> PWDT1_IN0
    PC7     --> PWDT1_IN1
    PC6     --> PWDT1_IN2
    */

    /* PWDT channel 0 */
    GPIO_SetFunc(GPIOC, GPIO_PIN8, GPIO_FUN3);
    /* PWDT channel 1 */
    GPIO_SetFunc(GPIOC, GPIO_PIN7, GPIO_FUN3);
    /* PWDT channel 2 */
    GPIO_SetFunc(GPIOC, GPIO_PIN6, GPIO_FUN3);

    pwdtConfig.overflowInterruptEn  = ENABLE;
    pwdtConfig.readyInterruptEn     = ENABLE;                                   /* enable data reday interrupt */
    pwdtConfig.interruptEn          = DISABLE;                                  /* Reserve for forward|reverse wind detection */
    pwdtConfig.mode                 = PWDT_MEASURE_MODE;                        /* enable PWDT module */
    pwdtConfig.edgeType             = PWDT_RISING_START_CAPTURE_ALL;            /* start to measure pulse from first rising-edge and all dege */
    pwdtConfig.clkPsc               = (PWDT_ClkPrescalerType)PWDT_CLK_PSC;      /* PWDT clock source presclaer is 8 */
    pwdtConfig.cmpModeEn            = ENABLE;                                   /* PWDT clock source is bus clock */
    pwdtConfig.hallModeEn           = ENABLE;
    pwdtConfig.filterEn             = ENABLE;                                   /* Enable PWDT Hall Interface */
    pwdtConfig.filterPsc            = (PWDT_FilterPrescalerType)PWDT_FILT_PSC;
    pwdtConfig.filterValue          = PWDT_FILT_VAL;
    pwdtConfig.callBack             = BLDC_Bemf_PWDTCallback;

    PWDT_Init(PWDT1, &pwdtConfig);
    NVIC_SetPriority(PWDT1_IRQn, 1);
}

/*!
* @brief PWDT Initialize in bldc hall control, config PWDT mode and PWDT interrupt.
*
* @param[in] none
* @return none
*/
void BLDC_Hall_PWDT_Initialize(void)
{
    PWDT_ConfigType  pwdtConfig;
    memset(&pwdtConfig, 0, sizeof(PWDT_ConfigType));

    /**
    GPIO Configuration for PWDT1
    PC8     --> PWDT1_IN0
    PC7     --> PWDT1_IN1
    PC6     --> PWDT1_IN2
    */

    /* PWDT channel 0 */
    GPIO_SetFunc(GPIOC, GPIO_PIN8, GPIO_FUN3);
    /* PWDT channel 1 */
    GPIO_SetFunc(GPIOC, GPIO_PIN7, GPIO_FUN3);
    /* PWDT channel 2 */
    GPIO_SetFunc(GPIOC, GPIO_PIN6, GPIO_FUN3);

    pwdtConfig.overflowInterruptEn   = ENABLE;
    pwdtConfig.readyInterruptEn      = ENABLE;                                   /* enable data reday interrupt */
    pwdtConfig.interruptEn           = ENABLE;                                   /* enable PWDT module interrupt */
    pwdtConfig.mode                  = PWDT_MEASURE_MODE;                        /* enable PWDT module */
    pwdtConfig.edgeType              = PWDT_FALLING_START_CAPTURE_ALL;           /* start to measure pulse from first rising-edge and all dege */
    pwdtConfig.clkPsc                = (PWDT_ClkPrescalerType)PWDT_CLK_PSC;      /* PWDT clock source presclaer is 6 */
    pwdtConfig.cmpModeEn             = DISABLE;                                  /* PWDT clock source is bus clock */
    pwdtConfig.hallModeEn            = ENABLE;
    pwdtConfig.filterEn              = ENABLE;                                   /* Enable PWDT Hall Interface */
    pwdtConfig.filterPsc             = (PWDT_FilterPrescalerType)PWDT_FILT_PSC;
    pwdtConfig.filterValue           = PWDT_FILT_VAL;
    pwdtConfig.callBack              = BLDC_Hall_PWDTCallback;

    PWDT_Init(PWDT1, &pwdtConfig);
    NVIC_SetPriority(PWDT1_IRQn, 1);
}

/*!
* @brief Peripheral Initialize in bldc App.
*
* @param[in] none
* @return none
*/
void BLDC_Peripheral_Initialize(void)
{
    MC_GPIO_Initialize();
    BLDC_TIM_Initialize();
    MC_DEBUG_PWM_Initialize(PWM0);
    BLDC_CTU_Init();
    BLDC_ADC_Init();
    BLDC_PWM_Initialize();
#if (defined BLDC_HALL)
    BLDC_Hall_PWDT_Initialize();
#elif (defined BLDC_SENSORLESS)
    BLDC_Bemf_TimerInit();
    #if (defined BEMF_DETECT_HARDWARE)
    BLDC_Bemf_ACMP_Init();
    BLDC_Bemf_PWDT_Initialize();
    #endif
#else

#endif
    NVIC_EnableIRQ(ADC0_IRQn);
}

/* =============================================  EOF  ============================================== */
