/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_ui_uart.c
*
* @brief This file provides UART communication with user interface of host computer functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac780x_gpio_reg.h"
#include "foc_ui_uart.h"
#include "foc_app.h"
#include "foc_keys.h"
#include "motor_debug.h"
#include "param_identify_api.h"
#include "version.h"
#include "foc_data_access_eflash.h"

/* ============================================  Define  ============================================ */
#define CMD_HEAD_BYTE                   0xAA                        /*!< Head byte in a package command */

#define CMD_CHECK_COMMUNICATION         0x00                        /*!< Command used to confirm communication */
#define CMD_GET_CODE_VERSION            0x01                        /*!< Command used to get code version */
#define CMD_START_MOTOR                 0x02                        /*!< Command used to start motor */
#define CMD_STOP_MOTOR                  0x03                        /*!< Command used to stop motor */
#define CMD_STOP_RAMP                   0x04                        /*!< Command used to stop ramp */
#define CMD_START_PARAM_IDENTIFY        0x05                        /*!< Command used to start motor param identify */
#define CMD_STOP_PARAM_IDENTIFY         0x06                        /*!< Command used to stop motor param identify */
#define CMD_MOTOR_FAULT_CLEAR           0x07                        /*!< Command used to clear fault */

#define CMD_FLASH_READ                  0x08                        /*!< Command used to read flash page to recovery data to variables */
#define CMD_FLASH_SAVE                  0x09                        /*!< Command used to save data to flash page */

#define CMD_START_HALLSELFLEARN         0x0A                        /*!< Command used to start motor hall sensor self learning */
#define CMD_STOP_HALLSELFLEARN          0x0B                        /*!< Command used to stop motor hall sensor self learning */

#define CMD_READ_REGISTER               0x10                        /*!< Command used to read param in register table from Mcu */
#define CMD_WRITE_REGISTER              0x11                        /*!< Command used to write param in register table to Mcu */
#define CMD_QUERY_PROCESS               0x12                        /*!< Command used to get command execute schedule from Mcu */

#define RECEIVE_TIM_MAXCNT              15000                       /*!< Recieved master data time count, execution time base is 2ms */
#define START_MOTOR_MAXCNT              1000                        /*!< Start motor time count, execution time base is 2ms */
#define START_IDENTIFY_MAXCNT           15000                       /*!< Identify process time count, execution time base is 2ms */


/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/* interactive buffer */
uint8_t g_uartRxBuf[RX_BUF_SIZE];
UART_BUF g_uiRxBuf = {RX_BUF_SIZE, 0, 0, 0, 0, g_uartRxBuf};        /*!< Define UI uart interactive rx buffer struct */

uint8_t g_uartTxBuf[TX_BUF_SIZE];
UART_BUF g_uiTxBuf = {TX_BUF_SIZE, 0, 0, 0, 0, g_uartTxBuf};        /*!< Define UI uart interactive tx buffer struct */

/* register */
uint8_t  *g_uiReg8Buf[UI_REG8_BUF_SIZE];                            /*!< Define UI uart interactive register pointer,point to g_uiCommParam one byte param */
uint16_t *g_uiReg16Buf[UI_REG16_BUF_SIZE];                          /*!< Define UI uart interactive register pointer,point to g_uiCommParam two byte param */
uint32_t *g_uiReg32Buf[UI_REG32_BUF_SIZE];                          /*!< Define UI uart interactive register pointer,point to g_uiCommParam four byte param */

UI_COMMUNICATION_PARAM g_uiCommParam;                               /*!< Define UI uart interactive param table */

/* communicate command execute */
CMD_PROC_STATUS g_cmdProcStatus ={0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};      /*!< Master command execute status */
CMD_PROC_VAR g_cmdProcVar;                                          /*!< Master command execute intermediate variable */

FOCLIB_VERSION              g_focLibVersion = {0};                  /*!< Struct to store lib version */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Mcu transmit a char to master through Uart.
*
* @param[in] uartTx: pointer to UART_BUF structure, store the data send out to master
* @param[in] data: the char to be send out to master
* @return none
*/
void UI_UART_PutChar(UART_BUF *uartTx, uint8_t data)
{
    UART_Type *localUartRegs = UART_GetUARTCtrl(UART1_INDEX);
    if ((*uartTx).uartBufCnt >= (*uartTx).uartBufSize)
    {
        (*uartTx).uartBufCnt = 0;
        (*uartTx).uartBufRdIndex = (*uartTx).uartBufWrIndex;
    }
    /* close interrrrupt */
    UART1->IER &= (~UART_IER_ETC_Msk);

    /* send first char */
    if ((*uartTx).uartBufCnt == 0)
    {
        if (UART_TxIsEmpty(UART1))
        {
            localUartRegs->RBR = (data & (0x00FF));
            /* open interrrrupt */
            UART1->IER |= UART_IER_ETC_Msk;
            return;
        }
    }

    *((*uartTx).uartBufAddr + (*uartTx).uartBufWrIndex) = data;
    if (++(*uartTx).uartBufWrIndex == (*uartTx).uartBufSize)
    {
        (*uartTx).uartBufWrIndex = 0;
    }
    ++(*uartTx).uartBufCnt;
    /* open interrrrupt */
    UART1->IER |= UART_IER_ETC_Msk;
}

/*!
* @brief UART read a char in current recieve buffer pointer,do not delete from buffer.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @param[in] num: recieced data ID in the recieve buffer
* @return a char read from the recieve buffer
*/
uint8_t UI_UART_ReadChar(UART_BUF *uartRx, uint16_t num)
{
    uint8_t data;
    uint16_t i;
    i = num;
    i += (*uartRx).uartBufRdIndex;
    if (i > (*uartRx).uartBufSize - 1)
    {
        i -= (*uartRx).uartBufSize;
    }
    data = *((*uartRx).uartBufAddr  + i);
    return data;
}

/*!
* @brief UART get a char in current recieve buffer pointer,and delete from buffer.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @return a char get from the recieve buffer
*/
uint8_t UI_UART_GetChar(UART_BUF *uartRx)
{
    uint8_t data;
    data = *((*uartRx).uartBufAddr + (*uartRx).uartBufRdIndex);
    if ((*uartRx).uartBufCnt != 0)
    {
        if (++(*uartRx).uartBufRdIndex == (*uartRx).uartBufSize)
        {
            (*uartRx).uartBufRdIndex = 0;
        }

        /* close interrrrupt, disable RXNE interrupt, ERXNE = 0 */
        UART_SetRXNEInterrupt(UART1, DISABLE);

        --(*uartRx).uartBufCnt;

        /* open interrrrupt, enable RXNE interrupt, ERXNE = 1 */
        UART_SetRXNEInterrupt(UART1, ENABLE);
    }
    return data;
}

/*!
* @brief UART delete a char in current recieve buffer pointer.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @param[in] num: the numbers of char to be deleted
* @return none
*/
void UI_UART_DeleteChar(UART_BUF *uartRx, uint16_t num)
{
    uint16_t i;
    if ((*uartRx).uartBufCnt >= num)
    {
        /* close interrrrupt */
        UART_SetRXNEInterrupt(UART1, DISABLE);

        (*uartRx).uartBufCnt -= num;

        /* open interrrrupt */
        UART_SetRXNEInterrupt(UART1, ENABLE);

        i = num;
        i += (*uartRx).uartBufRdIndex;

        if (i >= (*uartRx).uartBufSize)
        {
            i -= (*uartRx).uartBufSize;
        }
        (*uartRx).uartBufRdIndex = i;
    }
    else
    {
        /* close interrrrupt */
        UART_SetRXNEInterrupt(UART1, DISABLE);

        i = (*uartRx).uartBufCnt;
        (*uartRx).uartBufCnt = 0;

        /* open interrrrupt */
        UART_SetRXNEInterrupt(UART1, ENABLE);

        i += (*uartRx).uartBufRdIndex;
        if (i >= (*uartRx).uartBufSize)
        {
            i -= (*uartRx).uartBufSize;
        }
        (*uartRx).uartBufRdIndex = i;
    }
}

/*!
* @brief UART receive interrrupt isr,occur when receice a char from master.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_RxISR_Task(UART_BUF *uartRx)
{
    uint8_t data;

    /* ERXNE has been enable,and data has been recelved on rx pin */
    if (UART1->IER & UART_IER_ERXNE_Msk)
    {
        data = (uint8_t)UART_ReceiveData(UART1);
        if ((UART1->LSR0 & 0x0E) == 0)
        {
            g_uartRxBuf[g_uiRxBuf.uartBufWrIndex] = data;
            if (++(*uartRx).uartBufWrIndex == g_uiRxBuf.uartBufSize)
            {
                (*uartRx).uartBufWrIndex = 0;
            }
            if (++(*uartRx).uartBufCnt == (*uartRx).uartBufSize)
            {
                (*uartRx).uartBufCnt = 0;
                (*uartRx).uartBufOvf = 1;
                (*uartRx).uartBufRdIndex = 0;
                (*uartRx).uartBufWrIndex = 0;
            }
        }
        else
        {
//            UART_ReceiveData(UART1);
        }

        UART1->LSR0 |= (~UART_LSR0_DR_Msk);
    }
}

/*!
* @brief UART transmit interrrupt isr,occur when transmit a char to master.
*
* @param[in] uartTx: pointer to UART_BUF structure, store the data send out to master
* @return none
*/
void UI_UART_TxISR_Task(UART_BUF *uartTx)
{
    UART_Type * localUartRegs = UART_GetUARTCtrl(UART1_INDEX);

    /* ETC has been enable,and data has been transmitted from tx pin */
    if ((UART1->IER & UART_IER_ETC_Msk))
    {
        /* clear TC flag */
        UART1->LSR0 &= (~UART_LSR0_TC_Msk);
        if ((*uartTx).uartBufCnt)
        {
            --g_uiTxBuf.uartBufCnt;
            localUartRegs->RBR = g_uartTxBuf[g_uiTxBuf.uartBufRdIndex];
            if (++(*uartTx).uartBufRdIndex == (*uartTx).uartBufSize)
            {
                (*uartTx).uartBufRdIndex = 0;
            }
        }
        else
        {
            /* clear ETC */
            UART1->IER &= (~UART_IER_ETC_Msk);
        }
    }
}

/*!
* @brief Check the foc Lib version valid or not.
*
* @param[in] focLib: pointer to FOCLIB_VERSION structure, store the information to identify foc lib valid or not
* @return version valid information : 0 stand for valid, -1 stand for unvalid
*/
int8_t UI_CheckCodeVersionValid(FOCLIB_VERSION *focLib)
{
    if ((focLib->year < 2000) || (focLib->month > 12) || (focLib->day > 31) ||
    (MOTORAPP_VER_YEAR < 2000) || (MOTORAPP_VER_MONTH > 12) || (MOTORAPP_VER_DAY > 31))
    {
        return -1;
    }
    else
    {
        return 0;
    }
}


/*!
* @brief Uart communicate interrrupt isr,occur when exchange a char with master.
*
* @param[in] device: device pointer
* @param[in] wpara: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void UI_UART_IRQHandler(void *device, uint32_t wpara, uint32_t lpara)
{
    /*Rx interrupt*/
    UI_UART_RxISR_Task(&g_uiRxBuf);

    /*Tx interrupt*/
    UI_UART_TxISR_Task(&g_uiTxBuf);

}

/*!
* @brief Uart communication register table initialization.
*
* @param[in] none
* @return none
*/
void UI_RegBuf_Init(void)
{
//    static uint8_t read8Back_NULL = 0;
    static uint16_t read16Back_NULL = 0;
    static uint32_t read32Back_NULL = 0;

    memset(g_uiReg8Buf, 0U , sizeof(g_uiReg8Buf));
    memset(g_uiReg16Buf, 0U , sizeof(g_uiReg16Buf));
    memset(g_uiReg32Buf, 0U , sizeof(g_uiReg32Buf));
    memset(&g_uiCommParam, 0U, sizeof(g_uiCommParam));

    g_focLibVersion = FocLibVersionGet();

#if (defined HALL_SENSORS)
    g_uiCommParam.appMode = (uint16_t)SINE_HALL << 8;
#elif (defined ENCODER_HALL_SENSORS)
    g_uiCommParam.appMode = (uint16_t)SINE_HALL_ENCODER << 8;
#elif (defined ENCODER_SENSORS)
    g_uiCommParam.appMode = (uint16_t)SINE_ENCODER << 8;
#elif (defined SENSORLESS)
    g_uiCommParam.appMode = (uint16_t)SINE_SENSORLESS << 8;
    #if ((defined SMC_OBSERVE) && (defined ATAN_ANGLE_ESTIMATION))
    g_uiCommParam.appMode += (uint16_t)SMO_ATAN;
    #elif ((defined SMC_OBSERVE) && (defined PLL_ANGLE_ESTIMATION))
    g_uiCommParam.appMode += (uint16_t)SMO_PLL;
    #elif ((defined FLUX_OBSERVE) && (defined ATAN_ANGLE_ESTIMATION))
    g_uiCommParam.appMode += (uint16_t)FLUX_ATAN;
    #elif ((defined FLUX_OBSERVE) && (defined PLL_ANGLE_ESTIMATION))
    g_uiCommParam.appMode += (uint16_t)FLUX_PLL;
    #else
    #endif
#else
#endif

    g_uiCommParam.appVerYear = MOTORAPP_VER_YEAR;
    g_uiCommParam.appVerDate = (MOTORAPP_VER_MONTH << 8) + MOTORAPP_VER_DAY;
    g_uiCommParam.appVerMain = (MOTORAPP_MAIN_VER << 8) + MOTORAPP_SUB_VER;
    g_uiCommParam.appVerRelease = (MOTORAPP_REV_VER << 8) + MOTORAPP_BRANCH;
    g_uiCommParam.libVerYear = (uint16_t)g_focLibVersion.year;
    g_uiCommParam.libVerDate = ((uint16_t)g_focLibVersion.month << 8) + (uint16_t)g_focLibVersion.day;
    g_uiCommParam.libVerMain = ((uint16_t)g_focLibVersion.main << 8) + (uint16_t)g_focLibVersion.sub;
    g_uiCommParam.libVerRelease = ((uint16_t)g_focLibVersion.rev << 8) + (uint16_t)g_focLibVersion.branch;

    /* 8 Bit Zone*/
    g_uiReg8Buf[0] = (uint8_t*)&g_mcStatus;
    g_uiReg8Buf[1] = &defFocVars.curSampleMode;
    g_uiReg8Buf[2] = &defMotorParam.polePairs;
    g_uiReg8Buf[3] = &g_uiCommParam.dacOut1;
    g_uiReg8Buf[4] = &g_uiCommParam.dacOut2;
    g_uiReg8Buf[5] = &defOverModulation.overModEnable;
    g_uiReg8Buf[6] = (uint8_t*)&defFocVars.motorDir;
    g_uiReg8Buf[7] = &defFocVars.controlMode;
    g_uiReg8Buf[8] = &defFocVars.angleObtainWay;
    g_uiReg8Buf[9] = &defFocVars.voltObtainWay;
    g_uiReg8Buf[10] = &defFocVars.smoWay;
    g_uiReg8Buf[11] = &defFocVars.fullOrderSmoFilt;
    g_uiReg8Buf[12] = &g_hallSensor.hallStatus;
    g_uiReg8Buf[13] = &defHallSensor.hallCalcMode;
    g_uiReg8Buf[14] = &defHallSensor.hallSwapTrqCoff;
    g_uiReg8Buf[15] = &defHallSensor.hallSelfLearnMode;
    g_uiReg8Buf[16] = &defHallSensor.hallSelfLearnEnable;
    g_uiReg8Buf[17] = &defHallSensor.learnOrder;
    g_uiReg8Buf[18] = &g_hallSelfLearn.selfLearnOrder;
    g_uiReg8Buf[19] = &g_positionCommand.actionGear;
    g_uiReg8Buf[20] = &defPID.curPidSelfLearn;
    g_uiReg8Buf[21] = &defPID.speedPidSelfLearn;
    g_uiReg8Buf[22] = &defShortBrake.shortBrakeCurRatio;
    g_uiReg8Buf[23] = &g_shortBrake.subStatus;
    g_uiReg8Buf[24] = &defElecBrake.elecBrakeCurRatio;
    g_uiReg8Buf[25] = &defElecBrake.elecBrakeUdcMax;
    g_uiReg8Buf[26] = &g_pulseInject.pulseInjDoneFlag;
    g_uiReg8Buf[27] = &g_pulseInject.flowState;
    g_uiReg8Buf[28] = &g_pulseInject.subStatus;
    g_uiReg8Buf[29] = &g_powerDecrease.depFlag;
    g_uiReg8Buf[30] = &g_startUpCtrl.startUpFlag;
    g_uiReg8Buf[31] = &g_startUpCtrl.synFreReach;
    g_uiReg8Buf[32] = &defStartUp.curveSegNum;
    g_uiReg8Buf[33] = &defParamIdentify.identifyParSel;
    g_uiReg8Buf[34] = &defParamIdentify.statorResisOrderSel;
    g_uiReg8Buf[35] = &defParamIdentify.pmsmLdqTypeSel;

    /* 16 Bit Zone*/
    g_uiReg16Buf[0] = &g_uiCommParam.appMode;
    g_uiReg16Buf[1] = &g_adcTransform.vdcVt;
    g_uiReg16Buf[2] = (uint16_t*)&g_dcCurEst.dcCurTrue;
    g_uiReg16Buf[3] = (uint16_t*)&g_powerEst.powerInTrue;
    g_uiReg16Buf[4] = &g_uiCommParam.heatTemperature;
    g_uiReg16Buf[5] = (uint16_t*)&g_adcSampleReal.iaOffset;
    g_uiReg16Buf[6] = (uint16_t*)&g_adcSampleReal.ibOffset;
    g_uiReg16Buf[7] = (uint16_t*)&g_adcSampleReal.icOffset;
    g_uiReg16Buf[8] = (uint16_t*)&g_adcSampleReal.ibusOffset;
    g_uiReg16Buf[9] = &defCurRestruct.sampWinLength;
    g_uiReg16Buf[10] = &defCurRestruct.adSampTim;
    g_uiReg16Buf[11] = (uint16_t*)&defMotorParam.ratedVoltage;
    g_uiReg16Buf[12] = (uint16_t*)&defMotorParam.maxCur;
    g_uiReg16Buf[13] = (uint16_t*)&defMotorParam.maxFreq;
    g_uiReg16Buf[14] = (uint16_t*)&defMotorParam.statorRs;
    g_uiReg16Buf[15] = (uint16_t*)&defMotorParam.statorLs;
    g_uiReg16Buf[16] = (uint16_t*)&defMotorParam.ld;
    g_uiReg16Buf[17] = (uint16_t*)&defMotorParam.lq;
    g_uiReg16Buf[18] = &g_uiCommParam.appVerYear;
    g_uiReg16Buf[19] = &g_uiCommParam.appVerDate;
    g_uiReg16Buf[20] = &g_uiCommParam.appVerMain;
    g_uiReg16Buf[21] = &g_uiCommParam.appVerRelease;
    g_uiReg16Buf[22] = &g_uiCommParam.libVerYear;
    g_uiReg16Buf[23] = &g_uiCommParam.libVerDate;
    g_uiReg16Buf[24] = &g_uiCommParam.libVerMain;
    g_uiReg16Buf[25] = &g_uiCommParam.libVerRelease;
    g_uiReg16Buf[26] = &defFocVars.pwmDeadTime;
    g_uiReg16Buf[27] = &defDeadComp.cutoffCurVal;
    g_uiReg16Buf[28] = &defDeadComp.maxCompFreq;
    g_uiReg16Buf[29] = &defFocVars.pwmFreq;
    g_uiReg16Buf[30] = (uint16_t*)&g_speedCommand.speedTarget;
    g_uiReg16Buf[31] = (uint16_t*)&g_speedCommand.speedTargetRamp;
    g_uiReg16Buf[32] = (uint16_t*)&g_speedCommand.speedFbk;
    g_uiReg16Buf[33] = (uint16_t*)&g_speedCommand.avgSpeedFbkRpm;
    g_uiReg16Buf[34] = (uint16_t*)&g_speedCommand.speedMax;
    g_uiReg16Buf[35] = (uint16_t*)&g_speedCommand.speedMin;
    g_uiReg16Buf[36] = (uint16_t*)&defFocVars.motorAcceleration;
    g_uiReg16Buf[37] = (uint16_t*)&defFocVars.motorDeceleration;
    g_uiReg16Buf[38] = (uint16_t*)&defFocVars.stopSpeed;
    g_uiReg16Buf[39] = (uint16_t*)&g_speedCommand.speedTargetLimit;
    g_uiReg16Buf[40] = &defSCurve.accStartRatio;
    g_uiReg16Buf[41] = &defSCurve.accEndRatio;
    g_uiReg16Buf[42] = &defSCurve.decStartRatio;
    g_uiReg16Buf[43] = &defSCurve.decEndRatio;
    g_uiReg16Buf[44] = (uint16_t*)&g_asrSpeedEstCof.kpPu;
    g_uiReg16Buf[45] = (uint16_t*)&g_asrSpeedEstCof.kiPu;
    g_uiReg16Buf[46] = (uint16_t*)&defPll.coeffPll;
    g_uiReg16Buf[47] = (uint16_t*)&defSlideMode.kslide0;
    g_uiReg16Buf[48] = (uint16_t*)&defSlideMode.kslf0;
    g_uiReg16Buf[49] = (uint16_t*)&defSlideMode.maxErr0;
    g_uiReg16Buf[50] = (uint16_t*)&defSlideMode.kslide1;
    g_uiReg16Buf[51] = (uint16_t*)&defSlideMode.kslf1;
    g_uiReg16Buf[52] = (uint16_t*)&defSlideMode.maxErr1;
    g_uiReg16Buf[53] = (uint16_t*)&defSlideMode.fullOrderGain;
    g_uiReg16Buf[54] = (uint16_t*)&defFluxObserver.flitCoeff;
    g_uiReg16Buf[55] = (uint16_t*)&defHallSensor.learnCur;
    g_uiReg16Buf[56] = &defHallSensor.learnIntervalTime;
    g_uiReg16Buf[57] = &defHallSensor.learnDegree;
    g_uiReg16Buf[58] = &g_hallSelfLearn.selfLearnTheta;
    g_uiReg16Buf[59] = &g_hallSelfLearn.sectorWidthMax;
    g_uiReg16Buf[60] = &g_hallSelfLearn.sectorWidthMin;
    g_uiReg16Buf[61] = &g_encoderSensor.encoderCnt;
    g_uiReg16Buf[62] = (uint16_t*)&g_encoderSensor.encoderCntOffSet;
#if (defined MT_SPEED_MEASURE)
    g_uiReg16Buf[63] = (uint16_t*)&g_encoderSpeed.deltaPulse;
    g_uiReg16Buf[64] = &g_encoderSpeed.deltaTime;
    g_uiReg16Buf[65] = (uint16_t*)&g_encoderSpeed.measureFreq;
#else
    g_uiReg16Buf[63] = &read16Back_NULL;
    g_uiReg16Buf[64] = &read16Back_NULL;
    g_uiReg16Buf[65] = &read16Back_NULL;
#endif
    g_uiReg16Buf[66] = (uint16_t*)&g_positionCommand.timeMax;
    g_uiReg16Buf[67] = (uint16_t*)&defPID.fluxKp;
    g_uiReg16Buf[68] = (uint16_t*)&defPID.fluxKi;
    g_uiReg16Buf[69] = (uint16_t*)&defPID.fluxKd;
    g_uiReg16Buf[70] = (uint16_t*)&defPID.maxflux;
    g_uiReg16Buf[71] = (uint16_t*)&defPID.minflux;
    g_uiReg16Buf[72] = (uint16_t*)&defPID.torqueKp;
    g_uiReg16Buf[73] = (uint16_t*)&defPID.torqueKi;
    g_uiReg16Buf[74] = (uint16_t*)&defPID.torqueKd;
    g_uiReg16Buf[75] = (uint16_t*)&defPID.maxtorque;
    g_uiReg16Buf[76] = (uint16_t*)&defPID.mintorque;
    g_uiReg16Buf[77] = (uint16_t*)&defPID.speedKp;
    g_uiReg16Buf[78] = (uint16_t*)&defPID.speedKi;
    g_uiReg16Buf[79] = (uint16_t*)&defPID.speedKd;
    g_uiReg16Buf[80] = (uint16_t*)&defPID.maxSpeed;
    g_uiReg16Buf[81] = (uint16_t*)&defPID.minSpeed;
    g_uiReg16Buf[82] = (uint16_t*)&defPID.positionKp;
    g_uiReg16Buf[83] = (uint16_t*)&defPID.positionKi;
    g_uiReg16Buf[84] = (uint16_t*)&defPID.positionKd;
    g_uiReg16Buf[85] = (uint16_t*)&defPID.maxPosition;
    g_uiReg16Buf[86] = (uint16_t*)&defPID.minPosition;
    g_uiReg16Buf[87] = (uint16_t*)&defPID.curWicPu;
    g_uiReg16Buf[88] = (uint16_t*)&defPID.speedWic;
    g_uiReg16Buf[89] = &defPID.damp;
    g_uiReg16Buf[90] = (uint16_t*)&g_asrSpeedPIIn.refPu;
    g_uiReg16Buf[91] = (uint16_t*)&g_asrSpeedPIIn.fbkPu;
    g_uiReg16Buf[92] = (uint16_t*)&g_asrSpeedPIOut.outPu;
    g_uiReg16Buf[93] = &defFluxweak.voltageRef;
    g_uiReg16Buf[94] = (uint16_t*)&defFluxweak.min;
    g_uiReg16Buf[95] = (uint16_t*)&defFluxweak.maxModule;
    g_uiReg16Buf[96] = (uint16_t*)&defFluxweak.demagCur;
    g_uiReg16Buf[97] = (uint16_t*)&defFluxweak.nominalCur;
    g_uiReg16Buf[98] = (uint16_t*)&defFluxweak.kp;
    g_uiReg16Buf[99] = (uint16_t*)&defFluxweak.ki;
    g_uiReg16Buf[100] = (uint16_t*)&defLoadTorque.loadEstCoeff;
    g_uiReg16Buf[101] = (uint16_t*)&defLoadTorque.iqCompLimit;
    g_uiReg16Buf[102] = (uint16_t*)&g_torqueCommand.iqChangeStep;
    g_uiReg16Buf[103] = (uint16_t*)&g_torqueCommand.iqMax;
    g_uiReg16Buf[104] = (uint16_t*)&g_torqueCommand.iqMin;
    g_uiReg16Buf[105] = &defShortBrake.shortBrakeUdcMax;
    g_uiReg16Buf[106] = &defShortBrake.shortBrakeDoneFlagTs;
    g_uiReg16Buf[107] = (uint16_t*)&defShortBrake.shortQuitCur;
    g_uiReg16Buf[108] = &defElecBrake.brakeDepth1;
    g_uiReg16Buf[109] = &defElecBrake.brakeDepth2;
    g_uiReg16Buf[110] = (uint16_t*)&defElecBrake.elecBrakeFre1;
    g_uiReg16Buf[111] = (uint16_t*)&defElecBrake.elecBrakeFre2;
    g_uiReg16Buf[112] = (uint16_t*)&defElecBrake.isqAddStep;
    g_uiReg16Buf[113] = (uint16_t*)&defElecBrake.isqDecStep;
    g_uiReg16Buf[114] = &defElecBrake.elecBrakeDepth;
    g_uiReg16Buf[115] = &defElecBrake.elecBrakeMaxDepth;
#if (defined SAVE_ENERGY)
    g_uiReg16Buf[116] = &defEnergySave.maxCurChange;
    g_uiReg16Buf[117] = &defEnergySave.steadyCurChange;
    g_uiReg16Buf[118] = &defEnergySave.curChangeStep;
    g_uiReg16Buf[119] = (uint16_t*)&defEnergySave.maxIdTarget;
    g_uiReg16Buf[120] = (uint16_t*)&defEnergySave.minIdTarget;
    g_uiReg16Buf[121] = &defEnergySave.enterEnergySaveLoad;
#else
    g_uiReg16Buf[116] = &read16Back_NULL;
    g_uiReg16Buf[117] = &read16Back_NULL;
    g_uiReg16Buf[118] = &read16Back_NULL;
    g_uiReg16Buf[119] = &read16Back_NULL;
    g_uiReg16Buf[120] = &read16Back_NULL;
    g_uiReg16Buf[121] = &read16Back_NULL;
#endif
    g_uiReg16Buf[122] = &g_pulseInject.pwmTs;
    g_uiReg16Buf[123] = &g_pulseInject.busCur[0];
    g_uiReg16Buf[124] = &g_pulseInject.busCur[1];
    g_uiReg16Buf[125] = &g_pulseInject.busCur[2];
    g_uiReg16Buf[126] = &g_pulseInject.busCur[3];
    g_uiReg16Buf[127] = &g_pulseInject.busCur[4];
    g_uiReg16Buf[128] = &g_pulseInject.busCur[5];
#if (defined INIT_POS_DETEC)
    g_uiReg16Buf[129] = &defInitPosDetec.injVoltAmp;
    g_uiReg16Buf[130] = (uint16_t*)&g_initPosDetecParam.iaAmp;
    g_uiReg16Buf[131] = (uint16_t*)&g_initPosDetecParam.ibAmp;
    g_uiReg16Buf[132] = (uint16_t*)&g_initPosDetecParam.icAmp;
    g_uiReg16Buf[133] = &g_initPosDetecParam.estTheta0;
    g_uiReg16Buf[134] = &g_initPosDetecParam.estTheta;
#else
    g_uiReg16Buf[129] = &read16Back_NULL;
    g_uiReg16Buf[130] = &read16Back_NULL;
    g_uiReg16Buf[131] = &read16Back_NULL;
    g_uiReg16Buf[132] = &read16Back_NULL;
    g_uiReg16Buf[133] = &read16Back_NULL;
    g_uiReg16Buf[134] = &read16Back_NULL;
#endif
    g_uiReg16Buf[135] = &g_hfinj.hfiEstTheta;
    g_uiReg16Buf[136] = (uint16_t*)&g_hfinj.hfiEstFreq;
    g_uiReg16Buf[137] = (uint16_t*)&defHighFreqInj.hfiFreq;
    g_uiReg16Buf[138] = (uint16_t*)&defHighFreqInj.vPeak;
    g_uiReg16Buf[139] = (uint16_t*)&defHighFreqInj.pulseAmp;
    g_uiReg16Buf[140] = &defHighFreqInj.pulseTime;
    g_uiReg16Buf[141] = &defHighFreqInj.clearTime;
    g_uiReg16Buf[142] = (uint16_t*)&defHighFreqInj.highFreqThreshold;
    g_uiReg16Buf[143] = (uint16_t*)&defHighFreqInj.lowFreqThreshold;
    g_uiReg16Buf[144] = (uint16_t*)&defHighFreqInj.hfiVdFreqHigh;
    g_uiReg16Buf[145] = (uint16_t*)&defHighFreqInj.hfiVdFreqLow;
    g_uiReg16Buf[146] = (uint16_t*)&defHighFreqInj.hfiPllKp;
    g_uiReg16Buf[147] = (uint16_t*)&defHighFreqInj.hfiPllKi;
    g_uiReg16Buf[148] = (uint16_t*)&defHighFreqInj.bpfA0;
    g_uiReg16Buf[149] = (uint16_t*)&defHighFreqInj.bpfA2;
    g_uiReg16Buf[150] = (uint16_t*)&defHighFreqInj.bpfB2;
    g_uiReg16Buf[151] = (uint16_t*)&defHighFreqInj.lpfA0;
    g_uiReg16Buf[152] = (uint16_t*)&defHighFreqInj.lpfA1;
    g_uiReg16Buf[153] = (uint16_t*)&defHighFreqInj.lpfA2;
    g_uiReg16Buf[154] = (uint16_t*)&defHighFreqInj.lpfB2;
    g_uiReg16Buf[155] = (uint16_t*)&defHighFreqInj.thetaLpfCoeff;
    g_uiReg16Buf[156] = (uint16_t*)&defHighFreqInj.curLpfCoeff;
    g_uiReg16Buf[157] = &g_powerDecrease.depCoeff;
    g_uiReg16Buf[158] = &g_dcCurLimit.curLimitCoeff;
    g_uiReg16Buf[159] = &g_powerLimit.powerLimitCoeff;
    g_uiReg16Buf[160] = (uint16_t*)&defStartUp.spdSmoothCur;
    g_uiReg16Buf[161] = (uint16_t*)&defStartUp.synSpd;
    g_uiReg16Buf[162] = (uint16_t*)&defStartUp.curStartOffset;
    g_uiReg16Buf[163] = &defStartUp.alignTime;
    g_uiReg16Buf[164] = &defStartUp.alignTimeTwice;
    g_uiReg16Buf[165] = &defStartUp.alignCur;
    g_uiReg16Buf[166] = &defStartUp.alignCurTwice;
    g_uiReg16Buf[167] = &defStartUp.slopeDuration1;
    g_uiReg16Buf[168] = &defStartUp.slopeDuration2;
    g_uiReg16Buf[169] = &defStartUp.curveFreqStart;
    g_uiReg16Buf[170] = &defStartUp.curveCurStart;
    g_uiReg16Buf[171] = &defStartUp.curveVoltStart;
    g_uiReg16Buf[172] = &defStartUp.curveTime1;
    g_uiReg16Buf[173] = &defStartUp.curveTime2;
    g_uiReg16Buf[174] = &defStartUp.curveTime3;
    g_uiReg16Buf[175] = &defStartUp.curveTime4;
    g_uiReg16Buf[176] = &defStartUp.curveTime5;
    g_uiReg16Buf[177] = &defStartUp.curveFreqSet1;
    g_uiReg16Buf[178] = &defStartUp.curveFreqSet2;
    g_uiReg16Buf[179] = &defStartUp.curveFreqSet3;
    g_uiReg16Buf[180] = &defStartUp.curveFreqSet4;
    g_uiReg16Buf[181] = &defStartUp.curveFreqSet5;
    g_uiReg16Buf[182] = &defStartUp.curveCurSet1;
    g_uiReg16Buf[183] = &defStartUp.curveCurSet2;
    g_uiReg16Buf[184] = &defStartUp.curveCurSet3;
    g_uiReg16Buf[185] = &defStartUp.curveCurSet4;
    g_uiReg16Buf[186] = &defStartUp.curveCurSet5;
    g_uiReg16Buf[187] = &defStartUp.curveVoltSet1;
    g_uiReg16Buf[188] = &defStartUp.curveVoltSet2;
    g_uiReg16Buf[189] = &defStartUp.curveVoltSet3;
    g_uiReg16Buf[190] = &defStartUp.curveVoltSet4;
    g_uiReg16Buf[191] = &defStartUp.curveVoltSet5;
    g_uiReg16Buf[192] = &defParamIdentify.rsFirstInjectCurAmp;
    g_uiReg16Buf[193] = &defParamIdentify.rsSecondInjectCurAmp;
    g_uiReg16Buf[194] = &defParamIdentify.ldqCurLimit;
    g_uiReg16Buf[195] = &defParamIdentify.curDcSet;
    g_uiReg16Buf[196] = &defParamIdentify.curAcSetD;
    g_uiReg16Buf[197] = &defParamIdentify.curAcSetQ;
    g_uiReg16Buf[198] = &defParamIdentify.fluxIdentifySetFreq;
    g_uiReg16Buf[199] = &defParamIdentify.fluxIdentifySetCur;
    g_uiReg16Buf[200] = &read16Back_NULL;
    g_uiReg16Buf[201] = &read16Back_NULL;
    g_uiReg16Buf[202] = &read16Back_NULL;
    g_uiReg16Buf[203] = &read16Back_NULL;
    g_uiReg16Buf[204] = &read16Back_NULL;

    /* 32 Bit Zone*/
    g_uiReg32Buf[0] = &g_protector.ErrFlag.all;
    g_uiReg32Buf[1] = &defMotorParam.maxSpeed;
    g_uiReg32Buf[2] = (uint32_t*)&defMotorParam.bEmf;
    g_uiReg32Buf[3] = &defFluxObserver.compCofLimit;
    g_uiReg32Buf[4] = &defMrasObserver.cofAlpha;
    g_uiReg32Buf[5] = &defMrasObserver.cofBeta;
    g_uiReg32Buf[6] = (uint32_t*)&g_inertiaIdentify.inertiaSteadyValue;
#if (defined INIT_POS_DETEC)
    g_uiReg32Buf[7] = &defInitPosDetec.injVoltFreq;
#else
    g_uiReg32Buf[7] = &read32Back_NULL;
#endif
    g_uiReg32Buf[8] = (uint32_t*)&defHighFreqInj.bpfB1;
    g_uiReg32Buf[9] = (uint32_t*)&defHighFreqInj.lpfB1;
    g_uiReg32Buf[10] = (uint32_t*)&g_estBemfDebug;
    g_uiReg32Buf[11] = (uint32_t*)&g_bemfReliableThreshold;
    g_uiReg32Buf[12] = &defParamIdentify.curFreqSetD;
    g_uiReg32Buf[13] = &defParamIdentify.curFreqSetQ;
    g_uiReg32Buf[14] = &read32Back_NULL;
}

/*!
* @brief Uart return read register result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_RegRead_Result(void)
{
    uint8_t i = 0;
    uint8_t uiRead_lenth = 0;
    uint8_t uiRead_checkSum = 0;
    uint8_t uiRead8_returnTemp = 0;
    uint16_t uiRead16_returnTemp = 0;
    uint32_t uiRead32_returnTemp = 0;
    
    g_cmdProcVar.i8_num = ((g_cmdProcVar.uiRdBuf[1] & 0xF0) >> 4);
    g_cmdProcVar.i16_num = (g_cmdProcVar.uiRdBuf[1] & 0x0F);
    g_cmdProcVar.i32_num = g_cmdProcVar.uiRdBuf[0] - g_cmdProcVar.i8_num - g_cmdProcVar.i16_num;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiRead_checkSum += CMD_HEAD_BYTE;

    if (g_cmdProcStatus.readRegStatus == 0x02)
    {
        uiRead_lenth =  6;
        UI_UART_PutChar(&g_uiTxBuf, uiRead_lenth);
        uiRead_checkSum += uiRead_lenth;
        /* Motor ID */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
        uiRead_checkSum += g_cmdProcVar.motorID;
        /* read command */
        UI_UART_PutChar(&g_uiTxBuf, CMD_READ_REGISTER);
        uiRead_checkSum += CMD_READ_REGISTER;
        /* read result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.readRegStatus);
        uiRead_checkSum += g_cmdProcStatus.readRegStatus;
    }
    else if (g_cmdProcStatus.readRegStatus == 0x00)
    {
        uiRead_lenth =  2 * g_cmdProcVar.i8_num + 3 * g_cmdProcVar.i16_num + 5 * g_cmdProcVar.i32_num + 8;
        UI_UART_PutChar(&g_uiTxBuf, uiRead_lenth);
        uiRead_checkSum += uiRead_lenth;
        /* Motor ID */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
        uiRead_checkSum += g_cmdProcVar.motorID;
        /* read result */
        UI_UART_PutChar(&g_uiTxBuf, CMD_READ_REGISTER);
        uiRead_checkSum += CMD_READ_REGISTER;
        /* read result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.readRegStatus);
        uiRead_checkSum += g_cmdProcStatus.readRegStatus;
        /* total read ID numbers */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiRdBuf[0]);
        uiRead_checkSum += g_cmdProcVar.uiRdBuf[0];
        /* read ID type numbers */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiRdBuf[1]);
        uiRead_checkSum += g_cmdProcVar.uiRdBuf[1];
        if (g_cmdProcVar.i8_num)
        {
            for (i = 2; i <= (g_cmdProcVar.i8_num + 1); i++)
            {
                UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiRdBuf[i]);
                uiRead_checkSum += g_cmdProcVar.uiRdBuf[i];
                uiRead8_returnTemp = *g_uiReg8Buf[g_cmdProcVar.uiRdBuf[i]];
                UI_UART_PutChar(&g_uiTxBuf, (uiRead8_returnTemp & 0xFF));
                uiRead_checkSum += (uiRead8_returnTemp & 0xFF);
            }
        }
        if (g_cmdProcVar.i16_num)
        {
            for (i = (g_cmdProcVar.i8_num + 2); i <= (g_cmdProcVar.i8_num + g_cmdProcVar.i16_num + 1); i++)
            {
                UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiRdBuf[i]);
                uiRead_checkSum += g_cmdProcVar.uiRdBuf[i];
                uiRead16_returnTemp = *g_uiReg16Buf[(g_cmdProcVar.uiRdBuf[i] - UI_REG8_BUF_SIZE)];
                UI_UART_PutChar(&g_uiTxBuf, (uiRead16_returnTemp & 0xFF));
                uiRead_checkSum += (uiRead16_returnTemp & 0xFF);
                UI_UART_PutChar(&g_uiTxBuf, ((uiRead16_returnTemp >> 8) & 0xFF));
                uiRead_checkSum += ((uiRead16_returnTemp >> 8) & 0xFF);
            }
        }
        if (g_cmdProcVar.i32_num)
        {
            for (i = (g_cmdProcVar.i8_num + g_cmdProcVar.i16_num + 2); i <= (g_cmdProcVar.i8_num + g_cmdProcVar.i16_num + g_cmdProcVar.i32_num + 1); i++)
            {
                UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiRdBuf[i]);
                uiRead_checkSum += g_cmdProcVar.uiRdBuf[i];
                uiRead32_returnTemp = *g_uiReg32Buf[(g_cmdProcVar.uiRdBuf[i] - UI_REG8_BUF_SIZE - UI_REG16_BUF_SIZE)];
                UI_UART_PutChar(&g_uiTxBuf, (uiRead32_returnTemp & 0xFF));
                uiRead_checkSum += (uiRead32_returnTemp & 0xFF);
                UI_UART_PutChar(&g_uiTxBuf, ((uiRead32_returnTemp >> 8) & 0xFF));
                uiRead_checkSum += ((uiRead32_returnTemp >> 8) & 0xFF);
                UI_UART_PutChar(&g_uiTxBuf, ((uiRead32_returnTemp >> 16) & 0xFF));
                uiRead_checkSum += ((uiRead32_returnTemp >> 16) & 0xFF);
                UI_UART_PutChar(&g_uiTxBuf, ((uiRead32_returnTemp >> 24) & 0xFF));
                uiRead_checkSum += ((uiRead32_returnTemp >> 24) & 0xFF);
            }
        }
    }
    else
    {
        /* do nothing */
    }
    UI_UART_PutChar(&g_uiTxBuf, uiRead_checkSum);
}

/*!
* @brief Uart return write register result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_RegWrite_Result(void)
{
    uint8_t uiWrite_lenth = 6;
    uint8_t uiWrite_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiWrite_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiWrite_lenth);
    uiWrite_checkSum += uiWrite_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiWrite_checkSum += g_cmdProcVar.motorID;
    /* write command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_WRITE_REGISTER);
    uiWrite_checkSum += CMD_WRITE_REGISTER;
    /* write result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.writeRegStatus);
    uiWrite_checkSum += g_cmdProcStatus.writeRegStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiWrite_checkSum);

}

/*!
* @brief Uart return communicate result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_Communicate_Result(void)
{
    uint8_t uiCom_lenth = 6;
    uint8_t uiCom_checkSum = 0;

    /* refresh communicate status,prepare to answer master */
    g_cmdProcStatus.communicateStatus = 0x00;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiCom_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiCom_lenth);
    uiCom_checkSum += uiCom_lenth;
    /* no mater which motor,return 0 in shake hand communication */
    UI_UART_PutChar(&g_uiTxBuf, 0);
    uiCom_checkSum += 0;
    /* communication command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_CHECK_COMMUNICATION);
    uiCom_checkSum += CMD_CHECK_COMMUNICATION;
    /* communication result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.communicateStatus);
    uiCom_checkSum += g_cmdProcStatus.communicateStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiCom_checkSum);

    /* deinit communicate status to the init status */
    g_cmdProcStatus.communicateStatus = 0xFF;
}

/*!
* @brief Uart return get code version result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_GetVersion_Result(void)
{
    uint8_t uiGetVersion_lenth = 0;
    uint8_t uiGetVersion_checkSum = 0;
    int8_t codeValidState = -1;

    /* init getversion status */
    g_cmdProcStatus.getVersionStatus = 0xFF;
    codeValidState = UI_CheckCodeVersionValid(&g_focLibVersion);
    /* refresh getversion status,prepare to answer master */
    if (codeValidState)
    {
        /* version is unvalid ,get version fail */
        g_cmdProcStatus.getVersionStatus = 0x01;
    }
    else
    {
        /* version is valid ,get version success */
        g_cmdProcStatus.getVersionStatus = 0x00;
    }

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiGetVersion_checkSum += CMD_HEAD_BYTE;
    if (g_cmdProcStatus.getVersionStatus == 0x01)
    {
        uiGetVersion_lenth =  6;
        UI_UART_PutChar(&g_uiTxBuf, uiGetVersion_lenth);
        uiGetVersion_checkSum += uiGetVersion_lenth;
        /* no mater which motor,return 0 in get version communication */
        UI_UART_PutChar(&g_uiTxBuf, 0);
        uiGetVersion_checkSum += 0;
        /* get version command */
        UI_UART_PutChar(&g_uiTxBuf, CMD_GET_CODE_VERSION);
        uiGetVersion_checkSum += CMD_GET_CODE_VERSION;
        /* get version result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.getVersionStatus);
        uiGetVersion_checkSum += g_cmdProcStatus.getVersionStatus;
    }
    else if (g_cmdProcStatus.getVersionStatus == 0x00)
    {
        uiGetVersion_lenth =  22;
        UI_UART_PutChar(&g_uiTxBuf, uiGetVersion_lenth);
        uiGetVersion_checkSum += uiGetVersion_lenth;
        /* no mater which motor,return 0 in get version communication */
        UI_UART_PutChar(&g_uiTxBuf, 0);
        uiGetVersion_checkSum += 0;
        /* get version command */
        UI_UART_PutChar(&g_uiTxBuf, CMD_GET_CODE_VERSION);
        uiGetVersion_checkSum += CMD_GET_CODE_VERSION;
        /* get version result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.getVersionStatus);
        uiGetVersion_checkSum += g_cmdProcStatus.getVersionStatus;
        /* App version year */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[18]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[18]) & 0xFF);
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[18] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[18] >> 8) & 0xFF);
        /* App version mounth */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[19] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[19] >> 8) & 0xFF);
        /* App version day */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[19]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[19]) & 0xFF);
        /* App main version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[20] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[20] >> 8) & 0xFF);
        /* App sub version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[20]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[20]) & 0xFF);
        /* App release version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[21] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[21] >> 8) & 0xFF);
        /* App branch */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[21]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[21]) & 0xFF);
        /* Lib version year */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[22]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[22]) & 0xFF);
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[22] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[22] >> 8) & 0xFF);
        /* Lib version mounth */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[23] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[23] >> 8) & 0xFF);
        /* Lib version day */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[23]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[23]) & 0xFF);
        /* Lib main version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[24] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[24] >> 8) & 0xFF);
        /* Lib sub version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[24]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[24]) & 0xFF);
        /* Lib release version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[25] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[25] >> 8) & 0xFF);
        /* Lib branch */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiReg16Buf[25]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiReg16Buf[25]) & 0xFF);
    }
    else
    {
        /* do nothing */
    }
    UI_UART_PutChar(&g_uiTxBuf, uiGetVersion_checkSum);
}

/*!
* @brief Uart return start motor result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StartMotor_Result(void)
{
    uint8_t uiStartMotor_lenth = 6;
    uint8_t uiStartMotor_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStartMotor_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotor_lenth);
    uiStartMotor_checkSum += uiStartMotor_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiStartMotor_checkSum += g_cmdProcVar.motorID;
    /* start motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_MOTOR);
    uiStartMotor_checkSum += CMD_START_MOTOR;
    /* start motor result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.startMotorStatus);
    uiStartMotor_checkSum += g_cmdProcStatus.startMotorStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotor_checkSum);
}

/*!
* @brief Uart return stop motor result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopMotor_Result(void)
{
    uint8_t uiStopMotor_lenth = 6;
    uint8_t uiStopMotor_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStopMotor_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStopMotor_lenth);
    uiStopMotor_checkSum += uiStopMotor_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiStopMotor_checkSum += g_cmdProcVar.motorID;
    /* stop motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_MOTOR);
    uiStopMotor_checkSum += CMD_STOP_MOTOR;
    /* stop motor result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.stopMotorStatus);
    uiStopMotor_checkSum += g_cmdProcStatus.stopMotorStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStopMotor_checkSum);
}

/*!
* @brief Uart return stop ramp result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopRamp_Result(void)
{
    uint8_t uiStopRamp_lenth = 6;
    uint8_t uiStopRamp_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStopRamp_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStopRamp_lenth);
    uiStopRamp_checkSum += uiStopRamp_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiStopRamp_checkSum += g_cmdProcVar.motorID;
    /* stop motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_RAMP);
    uiStopRamp_checkSum += CMD_STOP_RAMP;
    /* stop motor result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.stopRampStatus);
    uiStopRamp_checkSum += g_cmdProcStatus.stopRampStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStopRamp_checkSum);
}

/*!
* @brief Uart return identify motor paramerters result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_Identify_Result(void)
{
    uint8_t uiIdentify_lenth = 6;
    uint8_t uiIdentify_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiIdentify_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentify_lenth);
    uiIdentify_checkSum += uiIdentify_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiIdentify_checkSum += g_cmdProcVar.motorID;
    /* parameter identification command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_PARAM_IDENTIFY);
    uiIdentify_checkSum += CMD_START_PARAM_IDENTIFY;
    /* parameter identification result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.identifyStatus);
    uiIdentify_checkSum += g_cmdProcStatus.identifyStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentify_checkSum);
}

/*!
* @brief Uart return start motor schedule to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StartMotor_Schedule(void)
{
    uint8_t uiStartMotorSche_lenth = 7;
    uint8_t uiStartMotorSche_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStartMotorSche_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotorSche_lenth);
    uiStartMotorSche_checkSum += uiStartMotorSche_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiStartMotorSche_checkSum += g_cmdProcVar.motorID;
    /* Query process command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_QUERY_PROCESS);
    uiStartMotorSche_checkSum += CMD_QUERY_PROCESS;
    /* start motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_MOTOR);
    uiStartMotorSche_checkSum += CMD_START_MOTOR;

    /* StartMotor Schedule */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.cmdProcStatus);
    uiStartMotorSche_checkSum += g_cmdProcStatus.cmdProcStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotorSche_checkSum);
}

/*!
* @brief Uart return identify motor paramerters schedule to master.
*
* @param[in] none
* @return none
*/
void UI_Return_Identify_Schedule(void)
{
    uint8_t uiIdentifySche_lenth = 0;
    uint8_t uiIdentifySche_checkSum = 0;
    uint16_t uiIdentifySche_duty = 0;
    float temp_Bemf = 0.0;
    uint16_t temp_BemfIntegerPart = 0;
    uint16_t temp_BemfDecimalPart = 0;

    uiIdentifySche_lenth = 21;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiIdentifySche_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentifySche_lenth);
    uiIdentifySche_checkSum += uiIdentifySche_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiIdentifySche_checkSum += g_cmdProcVar.motorID;
    /* Query process command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_QUERY_PROCESS);
    uiIdentifySche_checkSum += CMD_QUERY_PROCESS;
    /* identify Query ok */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.cmdProcStatus);
    uiIdentifySche_checkSum += g_cmdProcStatus.cmdProcStatus;
    /* identify command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_PARAM_IDENTIFY);
    uiIdentifySche_checkSum += CMD_START_PARAM_IDENTIFY;
    uiIdentifySche_duty = ParamIdentify_DutyCheck(&g_paramIdentify);
    /* paramidentify Schedule Low byte, step value */
    UI_UART_PutChar(&g_uiTxBuf, uiIdentifySche_duty);
    uiIdentifySche_checkSum += uiIdentifySche_duty;
    /* paramidentify Schedule High byte, duty value */
    UI_UART_PutChar(&g_uiTxBuf, ((uiIdentifySche_duty >> 8) & 0xFF));
    uiIdentifySche_checkSum += ((uiIdentifySche_duty >> 8) & 0xFF);

    temp_Bemf = g_paramIdentifyCtrl.bEmf;
    temp_BemfIntegerPart = (uint16_t)temp_Bemf;
    temp_BemfDecimalPart = (uint16_t)((temp_Bemf - temp_BemfIntegerPart) * 10000);

    /* RS Low byte */
    UI_UART_PutChar(&g_uiTxBuf, (g_paramIdentifyCtrl.rsId & 0xFF));
    uiIdentifySche_checkSum += (g_paramIdentifyCtrl.rsId & 0xFF);
    /* RS High byte */
    UI_UART_PutChar(&g_uiTxBuf, ((g_paramIdentifyCtrl.rsId >> 8) & 0xFF));
    uiIdentifySche_checkSum += ((g_paramIdentifyCtrl.rsId >> 8) & 0xFF);
    /* LD Low byte */
    UI_UART_PutChar(&g_uiTxBuf, (g_paramIdentifyCtrl.ld & 0xFF));
    uiIdentifySche_checkSum += (g_paramIdentifyCtrl.ld & 0xFF);
    /* LD High byte */
    UI_UART_PutChar(&g_uiTxBuf, ((g_paramIdentifyCtrl.ld >> 8) & 0xFF));
    uiIdentifySche_checkSum += ((g_paramIdentifyCtrl.ld >> 8) & 0xFF);
    /* LQ Low byte */
    UI_UART_PutChar(&g_uiTxBuf, (g_paramIdentifyCtrl.lq & 0xFF));
    uiIdentifySche_checkSum += (g_paramIdentifyCtrl.lq & 0xFF);
    /* LQ High byte */
    UI_UART_PutChar(&g_uiTxBuf, ((g_paramIdentifyCtrl.lq >> 8) & 0xFF));
    uiIdentifySche_checkSum += ((g_paramIdentifyCtrl.lq >> 8) & 0xFF);

    /* Bemf integer Low byte */
    UI_UART_PutChar(&g_uiTxBuf, (temp_BemfIntegerPart & 0xFF));
    uiIdentifySche_checkSum += (temp_BemfIntegerPart & 0xFF);
    /* Bemf integer High byte */
    UI_UART_PutChar(&g_uiTxBuf, ((temp_BemfIntegerPart >> 8) & 0xFF));
    uiIdentifySche_checkSum += ((temp_BemfIntegerPart >> 8) & 0xFF);
    /* Bemf decimal Low byte */
    UI_UART_PutChar(&g_uiTxBuf, (temp_BemfDecimalPart & 0xFF));
    uiIdentifySche_checkSum += (temp_BemfDecimalPart & 0xFF);
    /* Bemf decimal High byte */
    UI_UART_PutChar(&g_uiTxBuf, ((temp_BemfDecimalPart >> 8) & 0xFF));
    uiIdentifySche_checkSum += ((temp_BemfDecimalPart >> 8) & 0xFF);

    /* Inertia Low byte */
    UI_UART_PutChar(&g_uiTxBuf, 0x00);
    uiIdentifySche_checkSum += 0x00;
    /* Inertia High byte */
    UI_UART_PutChar(&g_uiTxBuf, 0x00);
    uiIdentifySche_checkSum += 0x00;

    /* reset init motor identify param */
    if (g_paramIdentifyCtrl.mainStep == 4)
    {
        g_paramIdentifyCtrl.initCtrl = 1;
    }

    UI_UART_PutChar(&g_uiTxBuf, uiIdentifySche_checkSum);
}

/*!
* @brief Uart return identify motor paramerters schedule to master.
*
* @param[in] none
* @return none
*/
void UI_Return_HallSelfLearn_Schedule(void)
{
    uint8_t uiHallSelfLearnSche_lenth = 0;
    uint8_t uiHallSelfLearnSche_checkSum = 0;
    uint16_t uiHallSelfLearnSche_duty = 0;

    uiHallSelfLearnSche_lenth = 17;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiHallSelfLearnSche_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiHallSelfLearnSche_lenth);
    uiHallSelfLearnSche_checkSum += uiHallSelfLearnSche_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiHallSelfLearnSche_checkSum += g_cmdProcVar.motorID;
    /* Query process command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_QUERY_PROCESS);
    uiHallSelfLearnSche_checkSum += CMD_QUERY_PROCESS;
    /* slef learn Query ok */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.cmdProcStatus);
    uiHallSelfLearnSche_checkSum += g_cmdProcStatus.cmdProcStatus;
    /* slef learn command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_HALLSELFLEARN);
    uiHallSelfLearnSche_checkSum += CMD_START_HALLSELFLEARN;

    uiHallSelfLearnSche_duty = HallSelfLearnDutyCheck(&g_hall);

    /* slef learn Schedule low byte */
    UI_UART_PutChar(&g_uiTxBuf, (uiHallSelfLearnSche_duty & 0xFF));
    uiHallSelfLearnSche_checkSum += (uiHallSelfLearnSche_duty & 0xFF);
    /* slef learn Schedule high byte */
    UI_UART_PutChar(&g_uiTxBuf, ((uiHallSelfLearnSche_duty >> 8) & 0xFF));
    uiHallSelfLearnSche_checkSum += ((uiHallSelfLearnSche_duty >> 8) & 0xFF);

    if (uiHallSelfLearnSche_duty != 25615)
    {
        /* selflearn order low byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
        /* selflearn order high byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;

        /* selflearn theta low byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
        /* selflearn theta high byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;

        /* selflearn sector width max low byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
        /* selflearn sector width max high byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;

        /* selflearn sector width min low byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
        /* selflearn sector width min high byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
    }
    else
    {
        /* selflearn order low byte */
        UI_UART_PutChar(&g_uiTxBuf, (g_hallSelfLearnBuffer.selfLearnOrder & 0xFF));
        uiHallSelfLearnSche_checkSum += (g_hallSelfLearnBuffer.selfLearnOrder & 0xFF);
        /* selflearn order high byte */
        UI_UART_PutChar(&g_uiTxBuf, ((g_hallSelfLearnBuffer.selfLearnOrder>> 8) & 0xFF));
        uiHallSelfLearnSche_checkSum += ((g_hallSelfLearnBuffer.selfLearnOrder>> 8) & 0xFF);

        /* selflearn theta low byte */
        UI_UART_PutChar(&g_uiTxBuf, (g_hallSelfLearnBuffer.selfLearnTheta & 0xFF));
        uiHallSelfLearnSche_checkSum += (g_hallSelfLearnBuffer.selfLearnTheta & 0xFF);
        /* selflearn theta high byte */
        UI_UART_PutChar(&g_uiTxBuf, ((g_hallSelfLearnBuffer.selfLearnTheta >> 8) & 0xFF));
        uiHallSelfLearnSche_checkSum += ((g_hallSelfLearnBuffer.selfLearnTheta >> 8) & 0xFF);

        /* selflearn sector width max low byte */
        UI_UART_PutChar(&g_uiTxBuf, (g_hallSelfLearnBuffer.sectorWidthMax & 0xFF));
        uiHallSelfLearnSche_checkSum += (g_hallSelfLearnBuffer.sectorWidthMax & 0xFF);
        /* selflearn sector width max high byte */
        UI_UART_PutChar(&g_uiTxBuf, ((g_hallSelfLearnBuffer.sectorWidthMax >> 8) & 0xFF));
        uiHallSelfLearnSche_checkSum += ((g_hallSelfLearnBuffer.sectorWidthMax >> 8) & 0xFF);

        /* selflearn sector width min low byte */
        UI_UART_PutChar(&g_uiTxBuf, (g_hallSelfLearnBuffer.sectorWidthMin & 0xFF));
        uiHallSelfLearnSche_checkSum += (g_hallSelfLearnBuffer.sectorWidthMin & 0xFF);
        /* selflearn sector width min high byte */
        UI_UART_PutChar(&g_uiTxBuf, ((g_hallSelfLearnBuffer.sectorWidthMin >> 8) & 0xFF));
        uiHallSelfLearnSche_checkSum += ((g_hallSelfLearnBuffer.sectorWidthMin >> 8) & 0xFF);
    }

    /* stop hall self learn */
    if (g_hallSelfLearnFinishFlag == 1)
    {
        defHallSensor.hallSelfLearnEnable = DISABLE;
        g_hallSelfLearnFinishFlag = 0;
//        HallSelfLearnResultInit(&g_hallSelfLearnBuffer);
    }

    UI_UART_PutChar(&g_uiTxBuf, uiHallSelfLearnSche_checkSum);
}

/*!
* @brief Uart return clear fault result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_ClearFault_Result(void)
{
    uint8_t uiClearFault_lenth = 6;
    uint8_t uiClearFault_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiClearFault_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiClearFault_lenth);
    uiClearFault_checkSum += uiClearFault_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiClearFault_checkSum += g_cmdProcVar.motorID;
    /* clear fault command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_MOTOR_FAULT_CLEAR);
    uiClearFault_checkSum += CMD_MOTOR_FAULT_CLEAR;
    /* Clear Fault result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.clearFaultStatus);
    uiClearFault_checkSum += g_cmdProcStatus.clearFaultStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiClearFault_checkSum);
}

/*!
* @brief Uart return stop identify motor paramerters result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopIdentify_Result(void)
{
    uint8_t uiIdentifyOff_lenth = 6;
    uint8_t uiIdentifyOff_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiIdentifyOff_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentifyOff_lenth);
    uiIdentifyOff_checkSum += uiIdentifyOff_lenth;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiIdentifyOff_checkSum += g_cmdProcVar.motorID;
    /* stop paramidentify command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_PARAM_IDENTIFY);
    uiIdentifyOff_checkSum += CMD_STOP_PARAM_IDENTIFY;
    /* stop paramidentify result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.identifyOffStatus);
    uiIdentifyOff_checkSum += g_cmdProcStatus.identifyOffStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentifyOff_checkSum);
}

/*!
* @brief Uart return check err result to master,occur when check err.
*
* @param[in] none
* @return none
*/
void UI_ReturnCheckErr_Result(void)
{
    uint8_t uiCheckErr_lenth = 6;
    uint8_t uiCheckErr_checkSum = 0;

    switch (g_cmdProcVar.uiCheckCmd)
    {
        case CMD_CHECK_COMMUNICATION:
            g_cmdProcStatus.communicateStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.communicateStatus;
        break;

        case CMD_GET_CODE_VERSION:
            g_cmdProcStatus.getVersionStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.getVersionStatus;
        break;

        case CMD_START_MOTOR:
            g_cmdProcStatus.startMotorStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.startMotorStatus;
        break;

        case CMD_STOP_MOTOR:
            g_cmdProcStatus.stopMotorStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.stopMotorStatus;
        break;

        case CMD_STOP_RAMP:
            g_cmdProcStatus.stopRampStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.stopRampStatus;
        break;

        case CMD_START_PARAM_IDENTIFY:
            g_cmdProcStatus.identifyStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.identifyStatus;
        break;

        case CMD_STOP_PARAM_IDENTIFY:
            g_cmdProcStatus.identifyOffStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.identifyOffStatus;
        break;

        case CMD_START_HALLSELFLEARN:
            g_cmdProcStatus.startHallSelfLearnStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.startHallSelfLearnStatus;
        break;

        case CMD_STOP_HALLSELFLEARN:
            g_cmdProcStatus.stopHallSelfLearnStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.stopHallSelfLearnStatus;
        break;

        case CMD_MOTOR_FAULT_CLEAR:
            g_cmdProcStatus.clearFaultStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.clearFaultStatus;
        break;

        case CMD_READ_REGISTER:
            g_cmdProcStatus.readRegStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.readRegStatus;
        break;

        case CMD_WRITE_REGISTER:
            g_cmdProcStatus.writeRegStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.writeRegStatus;
        break;

        case CMD_QUERY_PROCESS:
            g_cmdProcStatus.writeRegStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.writeRegStatus;
        break;

        default:
        break;
    }

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiCheckErr_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiCheckErr_lenth);
    uiCheckErr_checkSum += uiCheckErr_lenth;
    /* motorID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiCheckErr_checkSum += g_cmdProcVar.motorID;
    /* command id while checksum err happens */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiCheckCmd);
    uiCheckErr_checkSum += g_cmdProcVar.uiCheckCmd;
    /* Check err result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.checkeErrStatus);
    uiCheckErr_checkSum += g_cmdProcStatus.checkeErrStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiCheckErr_checkSum);
}

/*!
* @brief Uart communication consume time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_UART_TimeOut_Cnt(void)
{
    if (g_cmdProcVar.uartTimeOutCntEn == 1)
    {
        g_cmdProcVar.uartTimeOutCnt++;
    }
    else if (g_cmdProcVar.uartTimeOutCntEn == 0)
    {
        g_cmdProcVar.uartTimeOutCnt = 0;
    }
    else
    {

    }
}

/*!
* @brief Uart judge whether start motor success or not.
*
* @param[in] none
* @return none
*/
void UI_JudgeStartMotor(void)
{
#if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
    if (defHallSensor.hallSelfLearnEnable == DISABLE)
    {
        if (Get_McStatus() == RUN)
        {
            /* start motor success */
            g_cmdProcStatus.startMotorStatus = 0x00;
            /* stop count time,prepare to answer master */
            g_cmdProcVar.cmdProcTimeOutCntEn = 0;
        }
        else
        {
            /* starting motor ,fail */
            g_cmdProcStatus.startMotorStatus = 0x01;
            /* out of start motor excute limit time,still not start ready */
            if (g_cmdProcVar.cmdProcTimeOutCnt > START_MOTOR_MAXCNT)
            {
                /* stop count time,prepare to answer master */
                g_cmdProcVar.cmdProcTimeOutCntEn = 0;
            }
        }
    }
    else
    {
        /* in hall self learn mode,starting motor fail */
        g_cmdProcStatus.startMotorStatus = 0x01;
        /* stop count time,prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }
#elif (defined ENCODER_SENSORS)
    if ((Get_McStatus() == ALIGN) || (Get_McStatus() == RUN))
    {
        /* start motor success */
        g_cmdProcStatus.startMotorStatus = 0x00;
        /* stop count time,prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }
    else
    {
        /* starting motor ,fail */
        g_cmdProcStatus.startMotorStatus = 0x01;
        /* out of start motor excute limit time,still not start ready */
        if (g_cmdProcVar.cmdProcTimeOutCnt > START_MOTOR_MAXCNT)
        {
            /* stop count time, prepare to answer master */
            g_cmdProcVar.cmdProcTimeOutCntEn = 0;
        }
    }
#elif (defined SENSORLESS)
    if ((Get_McStatus() == ANGLE_JUDGE) || (Get_McStatus() == ALIGN) || (Get_McStatus() == RUN))
    {
        /* start motor success */
        g_cmdProcStatus.startMotorStatus = 0x00;
        /* stop count time, prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }
    else
    {
        /* starting motor, fail */
        g_cmdProcStatus.startMotorStatus = 0x01;
        /* out of start motor excute limit time */
        if (g_cmdProcVar.cmdProcTimeOutCnt > START_MOTOR_MAXCNT)
        {
            /* stop count time, prepare to answer master */
            g_cmdProcVar.cmdProcTimeOutCntEn = 0;
        }
    }
#endif
    if (g_cmdProcVar.cmdProcTimeOutCntEn == 0)
    {
        UI_Return_StartMotor_Result();
    }
}

/*!
* @brief Uart judge whether identify motor paramerters execute success or not.
*
* @param[in] none
* @return none
*/
void UI_JudgeStartIdentify(void)
{
    if (Get_McStatus() == IDENTIFY)
    {
        /* identify start success, prepare to report schedule */
        g_cmdProcStatus.identifyStatus = 0x00;
        /* stop count time, prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }
    else
    {
        /* identify start fail */
        g_cmdProcStatus.identifyStatus = 0x01;
        /* stop count time, prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }

    if (g_cmdProcVar.cmdProcTimeOutCntEn == 0)
    {
        UI_Return_Identify_Result();
    }
}

/*!
* @brief master command process time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_MasterCmd_ProcCnt(void)
{
    if (g_cmdProcVar.cmdProcTimeOutCntEn != 0)
    {
        g_cmdProcVar.cmdProcTimeOutCnt++;
        if (g_cmdProcVar.cmdProcTimeOutCntEn == 1)
        {
            UI_JudgeStartMotor();
        }
        else if (g_cmdProcVar.cmdProcTimeOutCntEn == 2)
        {
            UI_JudgeStartIdentify();
        }
        else
        {

        }
    }
    else if (g_cmdProcVar.cmdProcTimeOutCntEn == 0)
    {
        g_cmdProcVar.cmdProcTimeOutCnt = 0;
    }
}

/*!
* @brief Tool Uart peripheral initialization.
*
* @param[in] none
* @return none
*/
void UI_UART_Initialize()
{
    UART_ConfigType uartConfig;
    memset(&uartConfig, 0, sizeof(UART_ConfigType));

    /*!
    * GPIO Configuration for UART1
    * PC2     --> UART1_Tx
    * PC3     --> UART1_Rx
    */

    /* UART Tx pin */
    GPIO_SetFunc(GPIOC, GPIO_PIN2, GPIO_FUN1);
    /* UART Rx pin */
    GPIO_SetFunc(GPIOC, GPIO_PIN3, GPIO_FUN1);

    uartConfig.baudrate = TOOL_UART_BAUDRATE;
    uartConfig.parity = UART_PARI_NO;
    uartConfig.dataBits = UART_WORD_LEN_8BIT;
    uartConfig.stopBits = UART_STOP_1BIT;
    uartConfig.fifoByteEn = ENABLE;
    uartConfig.dmaEn = UART_DMA_TXRX_NONE;
    uartConfig.callBack = UI_UART_IRQHandler;
    /* set Uart1 config */
    UART_Init(UART1, &uartConfig);

    /* clear TC, TC_FLAG = 0 */
    UART1->LSR0 &= (~UART_LSR0_TC_Msk);
    /* enable TC interrupt, ETC = 1 */
    UART1->IER |= UART_IER_ETC_Msk;
    /* enable RXNE interrupt, ERXNE = 1 */
    UART_SetRXNEInterrupt(UART1, ENABLE);

    NVIC_ClearPendingIRQ(UART1_IRQn);
    NVIC_EnableIRQ(UART1_IRQn);
    NVIC_SetPriority(UART1_IRQn, 3);

}

/*!
* @brief Tool Uart peripheral initialization.
*
* @param[in] none
* @return none
*/
void UI_ToolUart_Init()
{
    /* Tool uart peripheral init */
    UI_UART_Initialize();

    /* Tool uart communication register table init */
    UI_RegBuf_Init();
}

/*!
* @brief Master start motor through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StartMotor(void)
{
    if (g_cmdProcVar.motorID & 0x0F)
    {
    /* init start motor status */
    g_cmdProcStatus.startMotorStatus = 0xFF;
    if ((SHORT_BRAKE_CTRL & 0x01) && (g_uiCommParam.appMode & 0x0300))
    {
		PushIn_McBufStatus(START_BRAKE);
    }
    else
    {
        PushIn_McBufStatus(START);
    }
    /* disable param identify */
    g_paramIdentifyCtrl.identifyCtrl = (uint8_t)DISABLE;
    /* start start motor excute time count */
    g_cmdProcVar.cmdProcTimeOutCntEn = 1;
}
    else if (g_cmdProcVar.motorID & 0xF0)
    {
        /* Start motor2 action */
    }
    else
    {
    }
}

/*!
* @brief Master stop motor through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopMotor(void)
{
    if (g_cmdProcVar.motorID & 0x0F)
    {
        /* init stop motor status */
        g_cmdProcStatus.stopMotorStatus = 0xFF;
        if (g_uiCommParam.appMode & 0x0400)
        {
            if (defHallSensor.hallSelfLearnEnable == ENABLE)
            {
                HallSensor_Init(PWDT_GetHallStatus(PWDT1), &g_hall, &g_focVars);
                PushIn_McBufStatus(STOP);
            }
            else if (STOP_MODE == SLOW_DOWN_STOP)
            {
                PushIn_McBufStatus(SLOW_DOWN);
            }
            else if (SHORT_BRAKE_CTRL > 1)
            {
                PushIn_McBufStatus(STOP_BRAKE);
            }
            else
            {
                PushIn_McBufStatus(STOP);
            }
        }
        else
        {
            PushIn_McBufStatus(STOP);
        }
        g_paramIdentifyCtrl.identifyCtrl = (uint8_t)DISABLE;
        /* stop motor success, prepare to answer master */
        g_cmdProcStatus.stopMotorStatus = 0x00;
        UI_Return_StopMotor_Result();
    }
    else if (g_cmdProcVar.motorID & 0xF0)
    {
        /* Stop motor2 action */
    }
    else
    {
    }
}

/*!
* @brief Master stop motor ramp start through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopRamp(void)
{
    if (g_cmdProcVar.motorID & 0x0F)
    {
        /* init stop ramp status */
        g_cmdProcStatus.stopRampStatus = 0xFF;

        #if (defined SENSORLESS)
        if (g_startUpCtrl.startUpFlag == 6)
        {
            /* stop ramp */
            g_speedCommand.speedTarget = g_speedCommand.speedTargetRamp;
            /* stop ramp success, prepare to answer master */
            g_cmdProcStatus.stopRampStatus = 0x00;
        }
        #elif (defined HALL_SENSORS)
        if (g_hallSensor.swapstate  > 0)
        {
            /* stop ramp */
            g_speedCommand.speedTarget = g_speedCommand.speedTargetRamp;
            /* stop ramp success, prepare to answer master */
            g_cmdProcStatus.stopRampStatus = 0x00;
        }
        #else
        /* Encoder */
        if (Get_McStatus() == RUN)
        {
            /* stop ramp */
            g_speedCommand.speedTarget = g_speedCommand.speedTargetRamp;
            /* stop ramp success,prepare to answer master */
            g_cmdProcStatus.stopRampStatus = 0x00;
        }
        #endif

        else
        {
            /* set ramp param,but can not stop at startup,unvalid command */
            g_cmdProcStatus.stopRampStatus = 0x02;
        }

        UI_Return_StopRamp_Result();
    }
    else if (g_cmdProcVar.motorID & 0xF0)
    {
        /* Stop motor2 ramp action */
    }
    else
    {
    }
}

/*!
* @brief Master start motor parameters identify through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure, store motor paramerters
* @return none
*/
void UI_StartIdentify(UART_BUF *uiRxBuf, MOTOR_PARAM_PU *paramPu)
{
    uint16_t i = 0;

    if (g_cmdProcVar.motorID & 0x0F)
    {
        /* init param identify status */
        g_cmdProcStatus.identifyStatus = 0xFF;
        for (i = 0; i < IDENTIFY_BUF_SIZE; i++)
        {
            /* save identify param */
            g_cmdProcVar.uiIdentifyBuf[i] = UI_UART_GetChar(uiRxBuf);
        }
        /* refresh motor param set by master */
        /* parameters identify selection */
        g_paramIdentify.pParamIdentifyCfg->identifyParSel = g_cmdProcVar.uiIdentifyBuf[0];
        /* pole pairs num */
        paramPu->pairsb = g_cmdProcVar.uiIdentifyBuf[1];
        /* caculate elec base freq */
        g_focVars.pCfg->baseFreq = (g_cmdProcVar.uiIdentifyBuf[3] + (g_cmdProcVar.uiIdentifyBuf[4] << 8)) * paramPu->pairsb / 60;
        paramPu->fbHz = g_focVars.pCfg->baseFreq;
        /* Ibus */
        paramPu->ibAp = (g_cmdProcVar.uiIdentifyBuf[5] + (g_cmdProcVar.uiIdentifyBuf[6] << 8)) / 100;
        /* Vdc */
        paramPu->ubVt = (g_cmdProcVar.uiIdentifyBuf[7] + (g_cmdProcVar.uiIdentifyBuf[8] << 8)) / 100;

        if (Get_McStatus() == IDLE)
        {
            g_paramIdentifyCtrl.identifyCtrl = (uint8_t)ENABLE;
            /* turn to identify mode */
            PushIn_McBufStatus(IDENTIFY);
        }
        else
        {
            g_paramIdentifyCtrl.identifyCtrl = (uint8_t)DISABLE;
        }

        /* start param identify excute time count */
        g_cmdProcVar.cmdProcTimeOutCntEn = 2;
    }
    else if (g_cmdProcVar.motorID & 0xF0)
    {
        /* Start motor2 identify action */
    }
    else
    {
    }
}

/*!
* @brief Master query command process schedule through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_QuerySchedule(UART_BUF *uiRxBuf)
{
    uint8_t queryCmd = 0;
    if (g_cmdProcVar.motorID & 0x0F)
    {
        /* init query command excute status */
        g_cmdProcStatus.cmdProcStatus = 0xFF;
        /* get query command id */
        queryCmd = UI_UART_GetChar(uiRxBuf);
        switch (queryCmd)
        {
        case CMD_START_MOTOR:
//     UI_Return_StartMotor_Schedule();
        break;

        case CMD_START_PARAM_IDENTIFY:
        g_cmdProcStatus.cmdProcStatus = 0x00;
        /* query command identify,report schedule */
        UI_Return_Identify_Schedule();
        break;

        case CMD_START_HALLSELFLEARN:
        g_cmdProcStatus.cmdProcStatus = 0x00;
        /* query command hall self learn, report schedule */
        UI_Return_HallSelfLearn_Schedule();
        break;

        default:
        break;
        }
    }
    else if (g_cmdProcVar.motorID & 0xF0)
    {
        /* motor2 Query Schedule */
    }
    else
    {
    }
}

/*!
* @brief Master stop motor parameters identify through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopIdentify(void)
{
    if (g_cmdProcVar.motorID & 0x0F)
    {
        /* init stop param identify status */
        g_cmdProcStatus.identifyOffStatus = 0xFF;
        PushIn_McBufStatus(STOP);
        /* init motor identify param again */
        g_paramIdentifyCtrl.initCtrl = 1;

        Ldq_IdentifyConfigReset(&g_focVars);
        /* refresh stop param identify status,prepare to answer master */
        g_cmdProcStatus.identifyOffStatus = 0x00;
        UI_Return_StopIdentify_Result();
    }
    else if (g_cmdProcVar.motorID & 0xF0)
    {
        /* Stop motor2 identify */
    }
    else
    {
    }
}

/*!
* @brief Uart return flash operation result to master.
*
* @param[in] cmd: Tool command
* @return none
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
void UI_Return_FlashTask_Result(uint8_t cmd)
{
    uint8_t uiFlashResult_length = 6;
    uint8_t uiFlashResult_checkSum = 0;
    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiFlashResult_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiFlashResult_length);
    uiFlashResult_checkSum += uiFlashResult_length;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiFlashResult_checkSum += g_cmdProcVar.motorID;
    /* flash operation command */
    UI_UART_PutChar(&g_uiTxBuf, cmd);
    uiFlashResult_checkSum += cmd;
    /* flash operation result */
    UI_UART_PutChar(&g_uiTxBuf, g_flashTaskFinish);
    uiFlashResult_checkSum += g_flashTaskFinish;
    UI_UART_PutChar(&g_uiTxBuf, uiFlashResult_checkSum);
}
#endif

/*!
* @brief Master recovery data from flash through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] cmd: Tool command
* @return none
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
void UI_FlashRecoveryData(UART_BUF *uiRxBuf, uint8_t cmd)
{
    uint16_t tempCmd = 0;
    g_dataAccessFlash.recoveryEnable = 1;
    if (Get_McStatus() == IDLE)
    {
        PushIn_McBufStatus(FLASH_TASK);
        tempCmd = UI_UART_GetChar(uiRxBuf);
        if (tempCmd == 0)
        {
            g_dataAccessFlash.recoveryMode = 1;
        }
        else if (tempCmd == 1)
        {
            g_dataAccessFlash.recoveryMode = 2;
        }
        else
        {
            g_dataAccessFlash.recoveryMode = 0;
        }
    }
    UI_Return_FlashTask_Result(cmd);
}
#endif

/*!
* @brief Master save data to flash through UI interface.
*
* @param[in] cmd: Tool command
* @return none
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
void UI_FlashSaveData(uint8_t cmd)
{
    if (Get_McStatus() == IDLE)
    {
        g_dataAccessFlash.saveEnable = 1;
        g_dataAccessFlash.saveMode = 2;
        PushIn_McBufStatus(FLASH_TASK);
    }
    UI_Return_FlashTask_Result(cmd);
}
#endif

/*!
* @brief Uart return flash operation result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StartHallSelfLearn_Result(void)
{
    uint8_t uiStartHallSelfLearn_length = 6;
    uint8_t uiStartHallSelfLearnResult_checkSum = 0;
    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStartHallSelfLearnResult_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStartHallSelfLearn_length);
    uiStartHallSelfLearnResult_checkSum += uiStartHallSelfLearn_length;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiStartHallSelfLearnResult_checkSum += g_cmdProcVar.motorID;
    /* start hall selflearn command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_HALLSELFLEARN);
    uiStartHallSelfLearnResult_checkSum += CMD_START_HALLSELFLEARN;
    /* start hall selflearn result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.startHallSelfLearnStatus);
    uiStartHallSelfLearnResult_checkSum += g_cmdProcStatus.startHallSelfLearnStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStartHallSelfLearnResult_checkSum);
}

/*!
* @brief Uart return flash operation result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopHallSelfLearn_Result(void)
{
    uint8_t uiStopHallSelfLearnResult_length = 6;
    uint8_t uiStopHallSelfLearnResult_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStopHallSelfLearnResult_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStopHallSelfLearnResult_length);
    uiStopHallSelfLearnResult_checkSum += uiStopHallSelfLearnResult_length;
    /* Motor ID */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.motorID);
    uiStopHallSelfLearnResult_checkSum += g_cmdProcVar.motorID;
    /* stop hall selflearn command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_HALLSELFLEARN);
    uiStopHallSelfLearnResult_checkSum += CMD_STOP_HALLSELFLEARN;
    /* stop hall selflearn result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.stopHallSelfLearnStatus);
    uiStopHallSelfLearnResult_checkSum += g_cmdProcStatus.stopHallSelfLearnStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStopHallSelfLearnResult_checkSum);
}

/*!
* @brief Master start Hall self learning through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_StartHallSensorSelfLearn(UART_BUF *uiRxBuf)
{
    uint8_t modeSelect = 0;
    if (g_cmdProcVar.motorID & 0x0F)
    {
        g_cmdProcStatus.startHallSelfLearnStatus = 0xFF;

        modeSelect = UI_UART_GetChar(uiRxBuf);
        defHallSensor.hallSelfLearnMode = modeSelect;

        if (Get_McStatus() == IDLE)
        {
            defHallSensor.hallSelfLearnEnable = ENABLE;
            PushIn_McBufStatus(HALL_SELFLEARN);
            g_cmdProcStatus.startHallSelfLearnStatus = 0x00;
        }
        else
        {
            defHallSensor.hallSelfLearnEnable = DISABLE;
            g_cmdProcStatus.startHallSelfLearnStatus = 0x01;
        }

        UI_Return_StartHallSelfLearn_Result();
    }
    else if (g_cmdProcVar.motorID & 0xF0)
    {
        /* Start motor2 hall selflearn */
    }
    else
    {
    }
}

/*!
* @brief Master stop Hall self learning through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopHallSensorSelfLearn(void)
{
    if (g_cmdProcVar.motorID & 0x0F)
    {
        g_cmdProcStatus.stopHallSelfLearnStatus = 0xFF;
        defHallSensor.hallSelfLearnEnable = DISABLE;
        g_cmdProcStatus.stopHallSelfLearnStatus = 0x00;
        UI_Return_StopHallSelfLearn_Result();
    }
    else if (g_cmdProcVar.motorID & 0xF0)
    {
        /* Stop motor2 hall selflearn */
    }
    else
    {
    }
}

/*!
* @brief Master clear motor fault status through UI interface.
*
* @param[in] none
* @return none
*/
void UI_ClearFault(void)
{
    uint16_t i = 0;

    if (g_cmdProcVar.motorID & 0x0F)
    {
        /* init clear fault status */
        g_cmdProcStatus.clearFaultStatus = 0xFF;
        PushIn_McBufStatus(STOP);
        for (i = 0; i <= 30; i++)
        {
            /* clear all fault bit */
            Fault_Clear(i);
        }
        /* refresh clear fault status,prepare to answer master */
        g_cmdProcStatus.clearFaultStatus = 0x00;
        /* init motor identify param again */
        g_paramIdentifyCtrl.initCtrl = 1;
        g_cmdProcVar.WriteSpeedValid = 0;
        UI_Return_ClearFault_Result();
    }
    else if (g_cmdProcVar.motorID & 0xF0)
    {
        /* clear motor2 fault */
    }
    else
    {
    }
}

/*!
* @brief Master read register data through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] dataLen: the char numbers of a master command
* @return none
*/
void UI_readRegister(UART_BUF *uiRxBuf, uint8_t dataLen)
{
    uint8_t i = 0;

    /* init read register status */
    g_cmdProcStatus.readRegStatus = 0xFF;
    /* save read register param total num */
    g_cmdProcVar.uiRdBuf[0] = UI_UART_GetChar(uiRxBuf);

    if (g_cmdProcVar.uiRdBuf[0] < RW_BUF_SIZE)
    {
        /* save read register param type */
        g_cmdProcVar.uiRdBuf[1] = UI_UART_GetChar(uiRxBuf);
        for (i = 2; i <= (g_cmdProcVar.uiRdBuf[0] + 1); i++)
        {
            /* save all read ragister param master id */
            g_cmdProcVar.uiRdBuf[i] = UI_UART_GetChar(uiRxBuf);
        }
        /* refresh read ragister status,prepare to transmit register param to master */
        g_cmdProcStatus.readRegStatus = 0x00;
    }
    else
    {
        /* read register param num out of range,unvalid command */
        g_cmdProcStatus.readRegStatus = 0x02;
        /* init read register param num */
        g_cmdProcVar.uiRdBuf[0] = 0;
        /* Delete whole command except head and length */
        UI_UART_DeleteChar(uiRxBuf, dataLen - 2);
    }
    UI_Return_RegRead_Result();

    /* init read buff,prepare next read command */
    memset(g_cmdProcVar.uiRdBuf, 0U , sizeof(g_cmdProcVar.uiRdBuf));
}

/*!
* @brief Master write register data through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] dataLen: the char numbers of a master command
* @return none
*/
void UI_WriteRegister(UART_BUF *uiRxBuf, uint8_t dataLen)
{
    uint8_t char1 = 0, char2 = 0, char3 = 0, char4 = 0, i = 0;
    uint16_t buf16 = 0;
    uint32_t buf32 = 0;

    /* init write register status */
    g_cmdProcStatus.writeRegStatus = 0xFF;
    /* save write register param total num */
    g_cmdProcVar.uiWrBuf[0] = UI_UART_GetChar(uiRxBuf);

    if (g_cmdProcVar.uiWrBuf[0] < RW_BUF_SIZE)
    {
        /* save write register param type */
        g_cmdProcVar.uiWrBuf[1] = UI_UART_GetChar(uiRxBuf);
        g_cmdProcVar.i8_num = ((g_cmdProcVar.uiWrBuf[1] & 0xF0) >> 4);
        g_cmdProcVar.i16_num = (g_cmdProcVar.uiWrBuf[1] & 0x0F);
        g_cmdProcVar.i32_num = g_cmdProcVar.uiWrBuf[0] - g_cmdProcVar.i8_num - g_cmdProcVar.i16_num;

        if (g_cmdProcVar.i8_num)
        {
            for (i = 2; i <= (g_cmdProcVar.i8_num + 1); i++)
            {
                /* save write register param master id */
                g_cmdProcVar.uiWrBuf[i] = UI_UART_GetChar(uiRxBuf);
                /* receive one byte */
                char1 = UI_UART_GetChar(uiRxBuf);

                /* write param to pointer id address */
                *g_uiReg8Buf[g_cmdProcVar.uiWrBuf[i]] = char1;
            }
        }
        if (g_cmdProcVar.i16_num)
        {
            for (i = (g_cmdProcVar.i8_num + 2); i <= (g_cmdProcVar.i8_num + g_cmdProcVar.i16_num + 1); i++)
            {
                /* save write register param master id */
                g_cmdProcVar.uiWrBuf[i] = UI_UART_GetChar(uiRxBuf);
                /* receive two byte */
                char1 = UI_UART_GetChar(uiRxBuf);
                char2 = UI_UART_GetChar(uiRxBuf);
                /* splicing regiater param */
                buf16 = char1 + (char2 << 8);

                /* write param to the pointer id address */
                *g_uiReg16Buf[(g_cmdProcVar.uiWrBuf[i] - UI_REG8_BUF_SIZE)]  = buf16;
                if (g_speedCommand.speedAcceleration != (pFocVarsCfg->motorAcceleration / 10))
                {
                    g_speedCommand.speedAcceleration = (pFocVarsCfg->motorAcceleration / 10);
                }
                if (g_speedCommand.speedDeceleration != (pFocVarsCfg->motorDeceleration / 10))
                {
                    g_speedCommand.speedDeceleration = (pFocVarsCfg->motorDeceleration / 10);
                }
            }
        }
        if (g_cmdProcVar.i32_num)
        {
            for (i = (g_cmdProcVar.i8_num + g_cmdProcVar.i16_num + 2); i <= (g_cmdProcVar.i8_num + g_cmdProcVar.i16_num + g_cmdProcVar.i32_num + 1); i++)
            {
                /* save write register param master id */
                g_cmdProcVar.uiWrBuf[i] = UI_UART_GetChar(uiRxBuf);
                /* receive four byte */
                char1 = UI_UART_GetChar(uiRxBuf);
                char2 = UI_UART_GetChar(uiRxBuf);
                char3 = UI_UART_GetChar(uiRxBuf);
                char4 = UI_UART_GetChar(uiRxBuf);
                /* splicing regiater param */
                buf32 = char1 + (char2 << 8) + (char3 << 16) + (char4 << 24);

                /* write param to the pointer id address */
                *g_uiReg32Buf[(g_cmdProcVar.uiWrBuf[i] - UI_REG8_BUF_SIZE - UI_REG16_BUF_SIZE)]  = buf32;
            }
        }

        /* refresh write ragister status, prepare to answer master */
        g_cmdProcStatus.writeRegStatus = 0x00;
    }
    else
    {
        /* write register param num out of range,unvalid command */
        g_cmdProcStatus.writeRegStatus = 0x02;
        /* init write register param num */
        g_cmdProcVar.uiWrBuf[0] = 0;
        /* Delete whole command except head and length */
        UI_UART_DeleteChar(uiRxBuf, dataLen - 2);
    }

    UI_Return_RegWrite_Result();

    /* refresh DAC print channel 0 */
    g_dispCh0 = g_uiCommParam.dacOut1;
    /* refresh DAC print channel 1 */
    g_dispCh1 = g_uiCommParam.dacOut2;

    /* init write buff,prepare next write command */
    memset(g_cmdProcVar.uiWrBuf, 0U , sizeof(g_cmdProcVar.uiWrBuf));
}

/*!
* @brief Uart process command recieved from master according to communication protocol.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_ToolCmd_Proc(UART_BUF *uiRxBuf)
{
    uint8_t headByte, dataLength, cmdData;
    uint8_t checkSum = 0, masterCmd = 0;
    uint16_t getDataTimeOut = 0, cmdNum = 0;

    if (((*uiRxBuf).uartBufCnt) >= 2)
    {
        headByte = UI_UART_ReadChar(uiRxBuf, 0);
        if (headByte == CMD_HEAD_BYTE)
        {
            dataLength = UI_UART_ReadChar(uiRxBuf, 1);
            g_cmdProcVar.uartTimeOutCntEn = 1;
            getDataTimeOut = g_cmdProcVar.uartTimeOutCnt;

            if ((((*uiRxBuf).uartBufCnt) >= dataLength) && (getDataTimeOut <= RECEIVE_TIM_MAXCNT))
            {
                g_cmdProcVar.uartTimeOutCntEn = 0;
                for (cmdNum = 0; cmdNum < dataLength - 1; cmdNum++)
                {
                    cmdData = UI_UART_ReadChar(uiRxBuf, cmdNum);
                    checkSum += cmdData;
                    if (cmdNum == 2)
                    {
                        g_cmdProcVar.motorID = cmdData;
                    }
                    else if (cmdNum == 3)
                    {
                        g_cmdProcVar.uiCheckCmd = cmdData;
                    }
                    else
                    {
                    }
                }
                if (checkSum == UI_UART_ReadChar(uiRxBuf, cmdNum))
                {
                    /* Delete head, length and motorID */
                    UI_UART_DeleteChar(uiRxBuf, 3);
                    while (dataLength > 0)
                    {
                        masterCmd = UI_UART_GetChar(uiRxBuf);

                        switch (masterCmd)
                        {
                            case CMD_CHECK_COMMUNICATION:
                                UI_Return_Communicate_Result();
                            break;

                            case CMD_GET_CODE_VERSION:
                                UI_Return_GetVersion_Result();
                            break;

                            case CMD_START_MOTOR:
                                UI_StartMotor();
                            break;

                            case CMD_STOP_MOTOR:
                                UI_StopMotor();
                            break;

                            case CMD_STOP_RAMP:
                                UI_StopRamp();
                            break;

                            case CMD_START_PARAM_IDENTIFY:
                                UI_StartIdentify(uiRxBuf, &g_motorParamPu);
                            break;

                            case CMD_QUERY_PROCESS:
                                UI_QuerySchedule(uiRxBuf);
                            break;

                            case CMD_STOP_PARAM_IDENTIFY:
                                UI_StopIdentify();
                            break;

                            case CMD_MOTOR_FAULT_CLEAR:
                                UI_ClearFault();
                            break;

                            #if (defined DATA_ACCESS_FLASH_ENABLE)
                            case CMD_FLASH_READ:
                                UI_FlashRecoveryData(uiRxBuf, CMD_FLASH_READ);
                            break;
                            #endif

                            #if (defined DATA_ACCESS_FLASH_ENABLE)
                            case CMD_FLASH_SAVE:
                                UI_FlashSaveData(CMD_FLASH_SAVE);
                            break;
                            #endif

                            case CMD_START_HALLSELFLEARN:
                                UI_StartHallSensorSelfLearn(uiRxBuf);
                            break;

                            case CMD_STOP_HALLSELFLEARN:
                                UI_StopHallSensorSelfLearn();
                            break;

                            case CMD_READ_REGISTER:
                                UI_readRegister(uiRxBuf, dataLength);
                            break;

                            case CMD_WRITE_REGISTER:
                                UI_WriteRegister(uiRxBuf, dataLength);
                            break;

                            default:
                            break;
                        }

                        /* Delete Check tail */
                        UI_UART_DeleteChar(uiRxBuf, 1);
                        return;
                    }
                }
                else
                {
                    /*  check err */
                    UI_ReturnCheckErr_Result();
                    /* Delete whole command */
                    UI_UART_DeleteChar(uiRxBuf, dataLength);
                }
            }
            else if ((((*uiRxBuf).uartBufCnt) < dataLength) && (getDataTimeOut > RECEIVE_TIM_MAXCNT))
            {
                g_cmdProcVar.uartTimeOutCntEn = 0;
                /* Delete Head and Length */
                UI_UART_DeleteChar(uiRxBuf, 2);
            }
        }
        /* Delete Head */
        UI_UART_DeleteChar(uiRxBuf, 1);
    }
}

/* =============================================  EOF  ============================================== */
